#if !defined RESOURCEMANAGER_H

#define RESOURCEMANAGER_H
#define TRUE 1
#define FALSE 0
#define BLOCK 0
#define NOTBLOCK 1

#define BLOCKED 0
#define ACTIVE 1

#define UNCOVERED  0
#define  COVERED 1

#include "Constants.h"
#include "train.h"
#include "TrackList.h"
#include "TrainList.h"

#include "MyDcc.h"
#include <afxwin.h>
#include <fstream>

using namespace std;


class ResourceManager:public CFrameWnd
{
public:

	ResourceManager();
/////////////////////////////////////////////////////////////////////////////
//Constructs a resourcemanager.  This is called when the user screeen is started
//and it creates a pointer to resoucemanager.  This will create a window that starts
//invisible, but fires timers to check if photocells are covered.  It initializes the
//ports for the turnouts and photocells useing functions defined in ccr.h and 
//coded in ccr.dll which is pointed to by the dp.lib.  Init the data structures
//of PreviousPCState and CurrentPCState arrays to be 0 to indicate that no photocells 
//have been triggered.  Also PhotocellManager array is init to 0 to represent that 
//no photocells bounding sections of track have been allocated.  Loads the track
//definition with a call to LoadTrack() and Sets a timer to fire every half second
////////////////////////////////////////////////////////////////////////////////	

	~ResourceManager();
////////////////////////////////////////////////////////////////////
//Destructor of Resouce Manager--Executed when a resource manager is detroyed
//close all the files we opened and kill the track to stop the trains
////////////////////////////////////////////////////////////////////////////

	int GetPhotoCellManager(int index);
/////////////////////////////////////////////////////////////
//returns the Current allocation of a particular photocell
//returns 0 if not allocated.  Returns Trains id that ownes the
// resource if it is allocated
///////////////////////////////////////////////////////////
	int GetTurnoutManager(int index);
///////////////////////////////////////////////////////////////////////////
//returns the current allocation of a particular photocell
//returns 0 if not allocated. Returns Train's id that owns the resource if
//the resource is allocated
///////////////////////////////////////////////////////////////////////////

	int AddTrainToTrack(Train * T);
//////////////////////////////////////////////////////////////////////////////
//This function is called from the UserScreen when the user wishes to add a train
//to the track.  Checks to make sure that the trains id greater than 0 and
//unique in the activetrainlist.  If Train id is bad function returns 0.  Then checks
//if the train can be added at present time by checking if the trains defined starting
//location is allocated to another train.  If the train cannot be added right now 
//function returns a 0.  If the train can be added to track it is added to the 
//activetrainlist and the photocellmanager array are updated to have the new Train
//allocated to its defined starting postion.  If all goes well function returns 1
/////////////////////////////////////////////////////////////////////////////////


	int DeleteTrainFromTrack(int id);
/////////////////////////////////////////////////////////////////////////////
//Deletes a train from the track and frees that trains resources
//Looks in both the activetrainlist and blocked list for the id of the train
//to delete and then searches the correct list. Find the train and adjusts
//the photocellmanager and turnout manager to 0 in the resources that the train
//owned prior to being deleted
//return 1 if successful
//returns 0 if not successful 
///////////////////////////////////////////////////////////////////////////
	int ToggleTurnout(int to);
//////////////////////////////////////////////////////////////////////////
//Toggles a specified turnout on the track if that turnout is not allocated
//to any train.  Calls the TO_Toggle(to) function defined in ccr.h and implemented
//in ccr.dll which is pointed to by the dp.lib
//Returns 0 if successful or the result of TO_Toggle(to) if successful
///////////////////////////////////////////////////////////////////////////////////
	int SetSpeed(int id, int newspeed);
//////////////////////////////////////////////////////////////////////////
//This Function Sets the speed of a train with the inputed id number  
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of SetSpeed()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
	int Stop(int id);
//////////////////////////////////////////////////////////////////////////
//This Function stops a train  with the inputed id number  
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of Stop()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
	int ChangeDirection(int id);
//////////////////////////////////////////////////////////////////////////
//This Function changes the direction of a train  with the inputed id number  
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of ChangeDirection()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
	int TurnHeadLightOn(int id, int on);
//////////////////////////////////////////////////////////////////////////
//This Function turns the headlight of a train to the on or off position
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of TurnHeadLightOn()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
	int ToggleHeadLight(int id);
//////////////////////////////////////////////////////////////////////////
//This Function toggles the headlight of a train 
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of ToggleHeadLight()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////

	int TurnTailLightOn(int id, int on);
//////////////////////////////////////////////////////////////////////////
//This Function turns the taillight of a train to the on or off position
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of TurnTailLightOn()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
	int ToggleTailLight(int id);
//////////////////////////////////////////////////////////////////////////
//This Function toggles the Taillight of a train 
//It finds the train in the active list checks to see if instructions are
//blocked to that train (trains are not allowed to get instructions when over
//a photocell between allocation and deallocation because they are under operating
//system control---Don't want them to stop or change direction while over a 
//photocell -- returns the value of ToggleTailLight()-- a train method defined in Train.h
//and implemented in train.cpp
//////////////////////////////////////////////////////////////////////////////////
	void ToggleEnabledCourse();
////////////////////////////////////////////////////////////////////////////
//This function toggles the course status from enabled to disabled or disabled to
//enables by issuing the DCC_reset() or DCC_kill() commands.
//If the function kills the track it stops all trains before issuing the kill command
//and toggles headlights and taillights to off. If the function enables the track
//it will reissue the last user speed command to all the trains on the active list
//////////////////////////////////////////////////////////////////////////////////
	bool GetTrackEnabled();
//////////////////////////////////////////////////////////////////////////////
//Returns the TrackEnabled Bool
////////////////////////////////////////////////////////////////////////////////
	int GetTurnoutDirection(int to);
///////////////////////////////////////////////////////////////////////////////////
//Returns the state of the inputed turnout -- wraps functions in ccr.dll
/////////////////////////////////////////////////////////////////////////////
	int GetPhotoCellStatus(int pc);
/////////////////////////////////////////////////////////////////////////////////
//Returns the Status of a photocell
//Covered = 1
//Uncovered = 0
//////////////////////////////////////////////////////////////////////////////////

private:

	void AllocateResources(Train * T, int pc, int to);
////////////////////////////////////////////////////////////////////////
//This function is called from UpdateResources() if that function found the
//the train requesting the resources of a photocell and turnout could indeed
//be allocated to the requesting train. This function updates the Oldphotocellhas
//and OldTurnoutHas so that when it comes time to deallocate this trains old
//resources we remember which ones in the array to put 0 values in.  The function
//then updates the Photocell that the train has to the one that it used to want
//next and updates the Photocellnext to be the one that the functions defined
//by tracklist to be the photocell it wanted next.  The function then fills the
//photocellmanager with the Trains Id and fills the turnoutmanager if the turnout
//is exists (ie not -1) with the trains id
//////////////////////////////////////////////////////////////////////////////////



	void DeallocateResources(Train * T);
///////////////////////////////////////////////////////////////////
//This function is called from HandleDeparture and set the PhotocellManager 
//and TurnoutMng back to 0 in the position defined by a trains OldphotocellHas
//Old TurnoutHas which are the resources that need to be deallocated when a 
//photocell becomes uncovered
///////////////////////////////////////////////////////////////////////
	int UpdateResources(Train * T, int pc, int to);
//////////////////////////////////////////////////////////////////////////
//Checks to see if the photocell and turnout that a train is requesting is allocated
//to another train already.  If both the Turnout and photocell are currently 
//not allocated then AllcoateResources is called to allocate the photocell and 
//turnout to the requesting train.  If the resources are not available FALSE 0 is
//returned.  This function will have racing conditions if timers can queue
////////////////////////////////////////////////////////////////////////////////////

	void LoadTrack();
/////////////////////////////////////////////////////////////////////////////////
//Loads the values into the array of ListLists CC and CW.  These data structures
//act as functions which output the next photocell and turnout that is associated
//with a photocell.  For example if CW[0] has a node (2, 1, STRAIGHT) it means that 
//if a train is heading Clockwise and hits photocell 0 then photocell 2 is connected
//to phtocell 0 through turnout 1 if turnout 1 is in a straight direction.
//Muliple nodes are added to the same index of the array when turnouts make it possible
//to travel to two different photocells.  In this case two separate nodes are added
//to the same index in CC or CW
///////////////////////////////////////////////////////////////////////////////////

    void HandleArrival(int photocell);
/////////////////////////////////////////////////////////////////////////
//Handles an Arrival for a photocell.  This is called from OnTimer() when a 
//photocells state goes from 0 to 1 signifying an arrival.  Fuction gets
//the id of the train from the photocellmanager.  This photocell must be owned
//by a train as their next photocell so get the id held in photocellmanager
//Get a pointer to the train using Find a TrainList function declared in TrainList.h
//and implemented in TrainList.cpp.  Set the LastPhotoCellCovered to be this photocell
//and set blocking instructions to BLOCK so that a train cannot recieve any instructions
//while over a photocell.  Use the TrackList indexed at inputed photocell to dind
//next photocell and turnout.  Update these resources if possible and if not possible
//stop the train and move it to the blocked list
///////////////////////////////////////////////////////////////////////////////////

	void HandleDeparture(int photocell);
//////////////////////////////////////////////////////////////////
//This function finds the train causing a departure and calls
//DeallocateResources to actually deallocate the trains resources
//set the trains BlockedInstructions back to NOTBLOCK so that
//the train can change direction again
///////////////////////////////////////////////////////////////////

	int TurnoutOpposite(int to);
/////////////////////////////////////////////////////////////////////////////////
//This function returns the opposite state of a turnout.  If the result of 
//Get_TO_Status(to) is STRAIGHT this fuction return CURVED.  If it is CURVED
//this returns STRAIGHT.  If neither it returns that the turnout is MOVING
//Get_TO_Status(int to) is defined in ccr.h
///////////////////////////////////////////////////////////////////////////
	void ActiveToBlocked(Train * T);
/////////////////////////////////////////////////////////////////////////////////
//This function Takes a Train and moves it from the active train list to the blocked
//train list.  It is called when a train cannot be allocated the resources that 
//it needs to continue and the train needs to be blocked  Called from HandleArrival()
//////////////////////////////////////////////////////////////////////////////////

	void BlockedToActive(Train * T);
/////////////////////////////////////////////////////////////////////////////////
//This function Takes a Train and moves it from the blocked train list to the active
//train list.  It is called when a train can be woken back up because the resources 
// it needs have now become available. 
//////////////////////////////////////////////////////////////////////////////////
	void ReleaseBlockedTrains(int ReleasedPhotocell, int ReleasedTurnout);
/////////////////////////////////////////////////////////////////////
//Searches the Blocked list to see if the inputed photocell number is
//the photocell that is blocking any of the trains on the blocked list
//if it finds a trian that is blocked by this resource it will release
//that train from the blocked list and move it to the activelist
//it will also reissue an arrival to the trains last covered photocell
//and a departure if the train is no longer covering that photocell to get 
//the correct values in the photocell and turnoutmng tables
///////////////////////////////////////////////////////////////////////



	void BatchToSwitch(Train * T, int pc, int to);
/////////////////////////////////////////////////////////////////////
//This function find the direction of a turnout given the next photocell
//to reach.  Called from UpdateResources and used to find the direction
//a turnout needs to be in using batch mode.  Since this is only called
//when we can update the resources, when we find the correct position
//of the turnouts we will toggle it
//////////////////////////////////////////////////////////////////////

	void TrainPrint(Train * T);
/////////////////////////////////////////////////////////////////////
//Print the contents of a train to the resource log
///////////////////////////////////////////////////////////////////


	void ResourceTablesPrint();
////////////////////////////////////////////////////////////////////////
//This function prints the contents of the PhotocellManger and TurnoutMng
//arrays to the resourceManager log file
/////////////////////////////////////////////////////////////////////////

	afx_msg int OnCreate(LPCREATESTRUCT lpcs);
/////////////////////////////////////////////////////////////////////////
//sets dimensions for a window incase ever want to show the resource manager 
//on the screen
//////////////////////////////////////////////////////////////////////////////

	afx_msg void OnTimer();
//////////////////////////////////////////////////////////////////////////////
//OnTimer is called everytime the timer set in the constructor of this class is
//fired or every half second.  This function keeps track of what photocells have
//changed states. (ie covered to uncovered and uncovered to covered) it then performs
//the operations to handle a trains arrival over that photocell or handle its departure 
//to a photocell.  Starts with a call to UpdateCurrentPCStatus() which reads all current
//values of the photocell states.  Next runs a loop to check if these new values
//are different from the values the last time the timer fired.  If CurrentState is greater
//then we had an arrival since covered is 1 and uncovered is 0.  If the PreviousPCState
//is greater we had a departture and the function calls HandleArrival() or HandleDeparture()
//to handle these situations ---- it then sets the PreviousPCState to be the 
//CurrentPCState to ready itself for the next call
/////////////////////////////////////////////////////////////////////////////////
	afx_msg void OnDestroy();
/////////////////////////////////////////////////////////////////////
//When the window is Destroyed kill the timer we created during the constructor
//of this class
/////////////////////////////////////////////////////////////////////////////
	void UpdateCurrentPCStatus();
/////////////////////////////////////////////////////////////////////////////////
//Updates the array CurrentPCState with the current values read from the photocell
//board
/////////////////////////////////////////////////////////////////////////////

	int PhotocellManager[MAXPHOTOCELLS];  
	//array that has id of train that owns the photocell of the index of 
	//the array--id of train for ownership 0 if not allocated

	int TurnoutMng[MAXTURNOUTS];
	//array that has id of train that owns the turnout of the index of 
	//the array--id of train for ownership 0 if not allocated

    TrackList CW[MAXPHOTOCELLS];
	//provides a function for Clockwise direction of a train--gives
	//next photocell, next photocell, and  the direction the turnouts
	//needs to be in

    TrackList CC[MAXPHOTOCELLS];
	//provides a function for CounterClockwise direction of a train--gives
	//next photocell, next photocell, and  the direction the turnouts
	//needs to be in

	int CurrentPCState[MAXPHOTOCELLS];
	//Current State of each photocell loaded in OnTimer

	int PreviousPCState[MAXPHOTOCELLS];
	//Previous State of each photocell from last time the timer fired

	TrainList ActiveTrains;
	//List of trains that are currently active

	TrainList BlockedTrains;
	//List of trains that are currently blocked

	bool TrackEnabled;  //status of the track (killed or not)

	ofstream Output; //output log filehandler

	CFont m_font; //font if ever want to show the resource manager window

	int m_cxChar; //average width of character -- used for window dimensions
	int m_cyChar; //average height of character -- used for window dimensions
	DECLARE_MESSAGE_MAP()

};

#endif



