// console.cpp

//*****************************************************************************
//     CONSOLE UTILITIES
//*****************************************************************************


#include <windows.h>  
#include <conio.h>

#include "console.h"

int GetTextColor( void )  {
	// return -1 on error
	
	int foreground;
	HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return -1;
	
    CONSOLE_SCREEN_BUFFER_INFO csbiInfo;
    if (!GetConsoleScreenBufferInfo(hs, &csbiInfo)) return 0;;
    foreground =  csbiInfo.wAttributes & 0x00F;
	return foreground;
}  // GetTextColor




bool SetTextColor(int foreground)  {
	WORD color;
	int background;
	HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;
	if(foreground < 0 || foreground > 15) return 0; 
	
	background = getbackgroundcolor();
	if (background == -1) return 0;
	background <<= 4;
	color = background | foreground;
	if (!SetConsoleTextAttribute(hs, color)) return 0;
	return 1;
}  // SetTextColor



int GetBackgroundColor( void )  {
	// return -1 on error
    
	int background;
	HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return -1;

    CONSOLE_SCREEN_BUFFER_INFO csbiInfo;
    if (!GetConsoleScreenBufferInfo(hs, &csbiInfo)) return 0;
    background =  csbiInfo.wAttributes & 0x00F0;
	background >>= 4;
	return background;
}  // GetBackgroundColor




bool SetBackgroundColor(int background)  {
	WORD color;
	int foreground;
	HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;
	if(background < 0 || background > 15) return 0;
    
	foreground = gettextcolor();
	if (foreground == -1) return 0;
	background <<= 4;
	color = background | foreground;
	if (!SetConsoleTextAttribute(hs, color)) return 0;
	return 1;
}  // SetBackgroundColor



bool GetCursorPosition(int &X, int &Y) {
	HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;
  
    CONSOLE_SCREEN_BUFFER_INFO csbiInfo;
    if (!GetConsoleScreenBufferInfo(hs, &csbiInfo)) return 0;;
   
    X =  csbiInfo.dwCursorPosition.X; 
    Y =  csbiInfo.dwCursorPosition.Y; 
	return 1;
} //GetCursorPosition


bool SetCursorOn(void)  {
    HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;
    CONSOLE_CURSOR_INFO  lpConsoleCursorInfo;   // address of cursor information 
    if (!GetConsoleCursorInfo(hs, &lpConsoleCursorInfo)) return 0 ;

    if (!lpConsoleCursorInfo.bVisible) {
		  lpConsoleCursorInfo.bVisible = 1;
		  if (!SetConsoleCursorInfo(hs, &lpConsoleCursorInfo)) return 0;
	}
	return 1;
}  // SetCursorOn 
 

bool ReadScreenChar( char & ch, int & background, int & foreground) {
    HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;
    
	int x,y;
	if(!GetCursorPosition(x,y)) return 0;

	char  Character;  // screen's char
	WORD  Attribute;        // address of buffer to receive attributes 
    DWORD  NumberRead;  // address of number of cells read from 
    COORD  dwReadCoord; // coordinates of first cell to read from 
    dwReadCoord.X = x;
	dwReadCoord.Y = y;   

   if (!ReadConsoleOutputAttribute(hs, &Attribute, 1, dwReadCoord, 
	&NumberRead)) return 0;
    foreground = Attribute & 0x000F;
	background = Attribute & 0x00F0; 
    background >>= 4;

   if (!ReadConsoleOutputCharacter(hs, &Character, 1, dwReadCoord,
	   &NumberRead)) return 0;      

   ch = Character;
   return 1;
}  //ReadScreenChar


bool SetCursorOff(void)  {
    HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;
    CONSOLE_CURSOR_INFO  lpConsoleCursorInfo;   // address of cursor information 
    if (!GetConsoleCursorInfo(hs, &lpConsoleCursorInfo)) return 0 ;

    if (lpConsoleCursorInfo.bVisible) {
		  lpConsoleCursorInfo.bVisible = 0;
		  if (!SetConsoleCursorInfo(hs, &lpConsoleCursorInfo)) return 0;
	}
	return 1;
}  // SetCursorOff
 



bool SetCursorPosition( int x, int y) {
	COORD  cor;

	HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	if (!hs) return 0;

	if (x <0 || x > 79) return 0;
	if (y <0 || y > 24) return 0;

    cor.X = x; cor.Y = y;
    if (!SetConsoleCursorPosition(hs, cor)) return 0;

	return 1;	
} // SetCursor



bool ClrScr( void ) {
   WORD  color;
   COORD coord;
   HANDLE hStdout;
   unsigned long cWritten;
   bool fSuccess;
   DWORD size;
   CONSOLE_SCREEN_BUFFER_INFO csbi;
   
   hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
   if (!hStdout) return 0;
   fSuccess = GetConsoleScreenBufferInfo(hStdout, &csbi);
   if (!fSuccess) return 0;

   color = csbi.wAttributes; 
   size = csbi.dwSize.X * csbi.dwSize.Y; // use existing size

   /* Fill a "size" character screen buffer with the space character. */ 
   coord.X = 0;            /* start at first cell */ 
   coord.Y = 0;            /*   of first row      */ 
 
   fSuccess = FillConsoleOutputCharacter( 
	hStdout,          /* screen buffer handle     */ 
	(TCHAR) ' ',      /* fill with spaces         */ 
	size,             /* number of cells to fill  */ 
	coord,            /* first cell to write to   */ 
	&cWritten);       /* actual number written to */ 
   if (!fSuccess) return 0;
   
   fSuccess = FillConsoleOutputAttribute( 
	hStdout,          /* screen buffer handle     */ 
	color,            /* color to fill with       */ 
	size,             /* number of cells to fill  */ 
	coord,            /* first cell to write to   */ 
	&cWritten);       /* actual number written to */ 
   if (!fSuccess) return 0;

   if (!SetCursorPosition(0,0) ) return 0;  //home the cursor
   if (!SetConsoleTextAttribute(hStdout, color)) return 0;  
   
   return 1;
 } // clrscr



bool SetWindowSize(int x, int y) {
   HANDLE hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
   CONSOLE_SCREEN_BUFFER_INFO csbiInfo;

   if ( !GetConsoleScreenBufferInfo(hStdout, &csbiInfo)) return 0;



SMALL_RECT srctWindow = csbiInfo.srWindow; 

if ( x < 0 || x > 79 ) return 0;
if ( y < 0 || y > 24 ) return 0;   


srctWindow.Top = 0;
srctWindow.Left = 0;
srctWindow.Right = x;;
srctWindow.Bottom = y;

if (!SetConsoleWindowInfo( 
	       hStdout,      // screen buffer handle       
	       TRUE,         // absolute coordinates        
	       &srctWindow)  // specifies new location      
    ) return 0;

return 1;
} // SetWindowSize


bool TextRectangle (int Left, int Top, int Right, int Bottom, 
					int background) {

     HANDLE hs = GetStdHandle(STD_OUTPUT_HANDLE);  
	 if (!hs) return 0;
   
	struct CHARINFO { // chi 
       char AsciiChar; 
       WORD Attributes;       // text and background colors
	};
	
    CHAR_INFO newbuffer[25*80];//  buffer with data to write  to rectangle
    COORD  newbuffersize;       // column-row size of newbuffer 
    COORD  newbuffercoord;      // upper-left cell of newbuffer (0,0) 
    SMALL_RECT  writeregion;    // pointer to rectangle to write to 
    
	if(background < 0 || background > 15) return 0;
 
	if (Left < 0 || Top < 0 || Right < 0 || Bottom < 0) return 0;
	if ( Right > 79 || Bottom > 24 ) return 0;
	if ( Left > Right ||  Top > Bottom) return 0;
	
    int r;
	WORD color;
	background <<= 4;
	color = background | 0 ; //foreground;
	for (r=0;r<25*80;r++) {
		   newbuffer[r].Char.AsciiChar = ' ';     // char/attr of the rectangle
		   newbuffer[r].Attributes = color;
	}

	newbuffersize.X = Right - Left + 1;     //size of the rectangle
    newbuffersize.Y = Bottom - Top + 1;

	newbuffercoord.X = 0;
	newbuffercoord.Y = 0;

	writeregion.Top = Top;       // position onscreen for rectangle
    writeregion.Bottom = Bottom;
	writeregion.Left = Left;
	writeregion.Right = Right;
    if (!WriteConsoleOutput(hs, newbuffer, newbuffersize, newbuffercoord, 
		& writeregion)) return 0;

	return 1;
}  // TextRectangle
   


int FlushInputBuffer(void)  {
	// Also returns the number of events found in the buffer
    // If unsuccessful, returns -1
	HANDLE hs = GetStdHandle(STD_INPUT_HANDLE);  
	if (!hs) return -1;
    
	DWORD n; //number of unread events
	if (!GetNumberOfConsoleInputEvents(hs, &n)) return -1;
	if (!FlushConsoleInputBuffer(hs)) return -1;
	return n;
} //FlushInputBuffer
	

char GetChFlush(void)  {
//****************************************
//*  Removes the char from the cin buffer
//*  after a call to getch()
//*   
//*  Does not appear to remove '\n' after
//*  calling cin >> int.  Use cin.get();
//****************************************
	char c;
	c = getch();
	FlushInputBuffer();
	return c;
}  // getchflush