//***********************************************************
// Module:  ccr.h
// Authors: Mark Sternig and DC Pankratz
// 1999
// 
// Description: Functions that initialize and access ports on a 210 io
// board from Industrial Computer Source Products (ICSP).
// Uses the ISA bus.
//
// Drivers for the board found in g:\pankdc\ccr\ioboard. 
// Runs on win98.
//*************************************************************


//io board base addresses and names
short const TO_output = 768;			//start port of TO output board  (HEX 303)
short const TO_input_straight = 772;		//start port of TO input A board (HEX 307)
short const TO_input_curved = 776;		//start port of TO input B board (HEX 30A)
short const Sensor_input_1 = 780;		//start port of Sensor input 1 board 
short const Sensor_input_2 = 784;		//start port of Sensor input 2 board 
short const Sensor_input_3 = 788;		//start port of Sensor input 3 board 
short const Sensor_input_4 = 792;		//start port of Sensor input 4 board 


//function prototypes


//***************************************************************//
//                                                               //
//                       BASE FUNCTIONS                          //
//                                                               //
//***************************************************************//


short _stdcall Open_Ports (void);
//***************************************************************
//Opens the Peek and Poke Driver.  Returns TRUE if successful.  
//This must be called before any calls are made to other modules.
//***************************************************************


void  _stdcall Close_Ports (void);
//***************************************************************
//Closes the driver. Should be called before the application
//exits.
//***************************************************************


short _stdcall Initialize_Board (short, short, short, short, short);
//***************************************************************
//Initailizes the board (1-9) to input and output sections based
//on the input parameters 0 and 1 for A, B, CL, and CH.
//Returns: 1 when successful and 0 when error
//
//              InitStatus = Initialize_Board(1, 0, 0, 0, 0);
//		InitStatus = Initialize_Board(2, 1, 1, 1, 1);
//		InitStatus = Initialize_Board(3, 1, 1, 1, 1);
//		InitStatus = Initialize_Board(4, 1, 1, 1, 1);
//***************************************************************




//short _stdcall Set_Chan_On (short, short);
//short _stdcall Set_Chan_Off (short, short);
//short _stdcall Get_Channel (short, short);
//short _stdcall Get_Port (short);
//short _stdcall Set_Port (short, short);
//short _stdcall TO_On (short);
//short _stdcall TO_Off (short);
//****************************************************************
// These are private functions used in ccr.cpp
//***************************************************************




//***************************************************************//
//                                                               //
//                       CCR FUNCTIONS                           //
//                                                               //
//***************************************************************//


short _stdcall TO_Toggle (short);
//***************************************************************
//Toggles a Turnout to the opposite position by reversing a relay. 
The turnout to switch is yhe input argument.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************


short _stdcall Get_TO_Status (short);
//***************************************************************
//Gets the position of a turnout.
//Returns:   straight, value = -1
//           curved,   value =  1
//           middle,   value =  0
//***************************************************************



short _stdcall Get_Sensor_Status (short);
//***************************************************************
//Gets the status of a optical sensor.
//Returns:    ON, value = 1
//           OFF, value = 0
//***************************************************************
