﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX.DirectInput;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;
using System.IO;

//***********************************Breakdown of Joy*************************************
//pretty simple, displays the devices of the system, gets you control of the one you want to use
//and uses the P/invoke to control the train and the track by manipulating the controller and its states
//by polling the controller on a timer.
//*****************************************************************************************

namespace CCR
{
    public partial class Joy : Form
    {

        [DllImport("tos_api.dll")]//Modifies train engine speed and direction
        public static extern bool TOS_SetTrainSpeedDCC(uint TrainAddress, uint Speed, byte direction, IntPtr pipe);
        //TOS_SetTrainSpeedDCC(unsigned long TrainAddress, unsigned long Speed, unsigned char direction, HANDLE pipe)

        [DllImport("tos_api.dll")]//Modifies train engine speed and direction
        public static extern bool TOS_SetTurnout(uint Address, byte value, IntPtr pipe);
        //TOS_SetTurnout(unsigned long Address, unsigned char value, HANDLE pipe)
        
        UInt16 trainaddr = 0;
        IntPtr pipeHandle;
        double oldSpeed = 0;
        int oldDir = -1;
        int newDir = -1;
        ListBox turnoutListbox;
        Device joystick;

        public Joy(UInt16 trainId, IntPtr pipe, ListBox turnOutCollection)  //This is done on form open
        {
            InitializeComponent();
            trainIDLabel.Text = "You are controlling train " + trainId.ToString();
            ptrHandle.Text = "Using pipe " + pipe.ToString() + " to DCCMan";
            //************************
            trainaddr = trainId;
            pipeHandle = pipe;
            turnoutListbox = turnOutCollection;
            //*************************
            pollTimer.Interval = 200;
            
            try
            {//this block grabs all devices on system and displays them for you to select
                DeviceList deviceList = Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly);
                
                    if(deviceList.Count > 0)
                    {
                        string[] systemJoysticks = new string[deviceList.Count];
                        int i = 0;
                        foreach (DeviceInstance deviceInstances in deviceList)
                        {
                            joystick = new Device(deviceInstances.InstanceGuid);
                            joystick.SetCooperativeLevel(this.Handle, CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive);
                            systemJoysticks[i] = joystick.DeviceInformation.InstanceName;
                            deviceListBox.Items.Add(systemJoysticks[i].ToString());
                            i++;
                        }
                        deviceListBox.SelectedIndex = 0;
                    }

             }
            catch (Exception ex)
            { MessageBox.Show(ex.ToString());}
        }
        

        private void deviceListBox_DoubleClick(object sender, EventArgs e)
        {   //this selects what device you plan on using..looks alot like above, but this actually acquires the control 
            //of the controlling device

            try
            {
                DeviceList deviceList = Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly);
                //^^^^^^^Creates a list of devices on system^^^^^^^^^^^^^^^^^^
               
                if (deviceList.Count > 0)
                {
                    string[] systemJoysticks = new string[deviceList.Count];
                    foreach (DeviceInstance deviceInstances in deviceList)
                    {
                        if (deviceListBox.SelectedIndex.ToString() == deviceInstances.InstanceName)
                        {//throws the value selected into a string and...
                            joystick = new Device(deviceInstances.InstanceGuid);//creates that instance for you
                            joystick.SetCooperativeLevel(this.Handle, CooperativeLevelFlags.Background | CooperativeLevelFlags.NonExclusive);
                        }
                    }
                }

                MessageBox.Show("Using " + joystick.DeviceInformation.InstanceName);

                joystick.SetDataFormat(DeviceDataFormat.Joystick);  //need to establish what type device
                joystick.Acquire(); //gets you control of device
                pollTimer.Start();  //starts polling the controller
            }
            catch (Exception ex)
            {MessageBox.Show(ex.ToString()); }
        }
    
        private void joystickSelectButton_Click(object sender, EventArgs e)
        {deviceListBox_DoubleClick(sender, e); }

        private void pollTimer_Tick(object sender, EventArgs e)
        {//manipulates the screen and the track/train according to the state of the controller
            int speedRatio = 7;
            label2.Text = "Normal Speed";
            label2.ForeColor = Color.Black;
            statusBox.Items.Clear();
            buttonStatusBox.Items.Clear();
            button1.Checked = false;
            button2.Checked = false;
            button3.Checked = false;
            button4.Checked = false;

            joystick.Poll();
            JoystickState state = joystick.CurrentJoystickState;    //an object that stores joystick state

            statusBox.Items.Add("x: " + state.X + " y: " + state.Y + " z: " + state.Z);//give some raw readings
            xStrengthBar.Value = state.X;
            if ((state.Z > 32768 && state.Z > state.Y) || (state.Z < 32767 && state.Z < state.Y))
                yzStrengthBar.Value = -1 * state.Z;
            else
                yzStrengthBar.Value = -1 * state.Y;

            buttonStatusBox.Items.Add(state.GetButtons()[0].ToString() + " " + state.GetButtons()[1].ToString() + " " + state.GetButtons()[2].ToString() + " " + state.GetButtons()[3].ToString());
            if (state.GetButtons()[0] > 0)
                button1.Checked = true;
            if (state.GetButtons()[1] > 0)
                button2.Checked = true;
            if (state.GetButtons()[2] > 0)
                button3.Checked = true;
            if (state.GetButtons()[3] > 0)
                button4.Checked = true;

            if ((state.GetButtons()[7] > 0) || (state.GetButtons()[6] > 0))
            {
                label2.Text = "Fast Speed";
                label2.ForeColor = Color.Red;
                speedRatio = 14;
            }


            //*********************** Change train Speed*****************************
            double speed = (double)yzStrengthBar.Value / (double)-32767;   //did negative to invert the trackbar..nothin else

            if (oldSpeed != speed)
            {
                oldSpeed = speed;
               // speedLabel.Text = speed.ToString();
                if (speed < 1)
                    TOS_SetTrainSpeedDCC(trainaddr, (uint)((1 - speed) * speedRatio), 1, pipeHandle);
                else
                    TOS_SetTrainSpeedDCC(trainaddr, (uint)((speed - 1) * speedRatio), 0, pipeHandle);
            }
            
            //************************Change track Direction************************

            newDir = -1;
            if (state.X < 32767)
                newDir = 0;
            else if (state.X > 32767)
                newDir = 1;
            if ((oldDir != newDir) && (newDir != -1))
            {
                speedLabel.Text = oldDir.ToString();
                for (int i = 0; i < turnoutListbox.Items.Count; i++)
                {
                    TOS_SetTurnout(Convert.ToUInt32(turnoutListbox.Items[i].ToString()), (byte)newDir, pipeHandle);
                    System.Threading.Thread.Sleep(167);
                    //Application.DoEvents();
                }
                oldDir = newDir;
            } 
        }
    }
}
