// CAddTrainDialog.cpp : implementation file
// bmcvey 6-26-01
// functionality of the dialog window to add a new train to the track
//    by requesting information, validating, passing back to parent
// the dialog is MODELESS .... can add multiple trains, can switch to
//    userscreen and dialog will remain until dismissed

// need to "create" dialog according to screen metrics .. currently uses menu.rc
// also, note that the train constructor does not allow for forward/reverse?????
#include "stdafx.h"
#include "TrackList.h"
#include "Constants.h"
#include "CAddTrainDialog.h"

#include <fstream>

using namespace std;

const int BATCH = 0;
const int INTERACTIVE = 1;
const int CLOCKWISE = 1;
const int COUNTERCLOCKWISE = 0;
const int FORWARD = 1;
const int REVERSE = 0;

const int ON = 1;
const int OFF = 0;
/////////////////////////////////////////////////////////////////////////////
// CAddTrainDialog dialog

CAddTrainDialog::CAddTrainDialog( CWnd* pParent)
	: CDialog(CAddTrainDialog::IDD, pParent)
{
	if (pParent == NULL) 
		ptrCV = AfxGetMainWnd();
	else
		ptrCV = pParent;

	TrainID = 0;

}

// only because i am so lazy today ...
const int CW = CLOCKWISE;
const int CC = COUNTERCLOCKWISE;

// Col 1:Photocell, Col 2: NextPhotoCell, COl 3:Direction,Col4:Turnout, COL5:Turnout Position
// this array can be completed by RMgr, passed to CUserScreen and finally here  whew!

int CAddTrainDialog::TrackInfo[MAX_TRACKINFO][5] = 
{   0, 2, CW, 1, STRAIGHT,
    1, 2, CW, 1, CURVED,
	2, 7, CW, -1, NOTURNOUT,
	3, 1, CW, 0, STRAIGHT,
	4, 12, CW, 6, MOVING,
	5, 4, CW, -1, NOTURNOUT,
	7, 15, CW, 2, STRAIGHT,
	8, 0, CW, -1, NOTURNOUT,
	9, 3, CW, -1, NOTURNOUT,
	11, 5, CW, 6, NOTURNOUT,
	12, 15, CW, 2, CURVED,
	13, 8, CW, 4, STRAIGHT,
	13, 9, CW, 4, CURVED,
	14, 11, CW, -1, NOTURNOUT,
	15, 13, CW, 3, STRAIGHT,
	15, 14, CW, 3, CURVED,

	0, 8, CC, -1, NOTURNOUT,
	1, 3, CC, 0, STRAIGHT,
	2, 1, CC, 1, CURVED,
	2, 0, CC, 1, STRAIGHT,
	3, 9, CC, -1, NOTURNOUT,
	4, 5, CC, -1, NOTURNOUT,
	5, 11, CC, 6, MOVING,
	7, 2, CC, -1, NOTURNOUT,
	8, 13, CC, 4, STRAIGHT,
	9, 13, CC, 4, CURVED,
	11, 14, CC, -1, NOTURNOUT,
	12, 4, CC, 6, MOVING,
	13, 15, CC, 3, STRAIGHT,
	14, 15, CC, 3, CURVED,
	15, 7, CC, 2, STRAIGHT,
	15, 12, CC, 2, CURVED
};

// passes information from/to data members and controls.
// invoked by InitDialog (from) and UpdateData (to)
void CAddTrainDialog::DoDataExchange(CDataExchange* pDX)
{	
	CDialog::DoDataExchange(pDX);
	//Mode assigned when radiobuttons clicked
	DDX_Text(pDX, IDC_MAXSPEED, MaxSpeed);
	DDX_Text(pDX, IDC_PCPASSEDCOMBO, PCHas);
	DDX_Text(pDX, IDC_NEXTPCCOMBO, NextPC);
	DDX_Text(pDX, IDC_TURNOUTCOMBO, TOHas);
//DX_Radio(pDx, 
	CString fn;
	DDX_Text(pDX, IDC_BATCHFILEEDIT, fn);
	strcpy(FileName, fn);
	
	DDX_Text(pDX, IDC_TRAINIDEDIT, TrainID);
	DDX_Text(pDX, IDC_TRAINNAMEEDIT, fn);
	strcpy(TrainName, fn);

	DDX_CBString(pDX, IDC_TRAINDIRCOMBO, fn);
	if (strcmp(fn, "Forward"))
		Direction = REVERSE;
	else
		Direction = FORWARD;

	DDX_CBString(pDX, IDC_TRACKDIRCOMBO, fn);
	if (strcmp(fn, "Clockwise"))
		DirAroundTrack = COUNTERCLOCKWISE;
	else
		DirAroundTrack = CLOCKWISE;
	
}

// called by InitDialog and OnClear and OnAdd since all need
// to return dialog controls to default values
void CAddTrainDialog::SetDefaults()
{
	// always begin with new train selected and empty ID and name
	CButton* p = (CButton*)GetDlgItem(IDC_EXISTINGTRAIN);
	p->SetCheck(BST_UNCHECKED);
	p = (CButton*)GetDlgItem(IDC_NEWTRAIN);
	p->SetCheck(BST_CHECKED);


	CComboBox* q = (CComboBox*)GetDlgItem(IDC_EXISTINGTRAINLIST);
	CEdit* qID = (CEdit*)GetDlgItem(IDC_TRAINIDEDIT);
	CEdit* qName = (CEdit*)GetDlgItem(IDC_TRAINNAMEEDIT);

	q->SetCurSel(0);
	q->EnableWindow(FALSE);
	qID->EnableWindow(TRUE);
	qID->SetWindowText("");
	qName->EnableWindow(TRUE);
	qName->SetWindowText("");

	// set range for speed spinbutton with initial selection 0
	CSpinButtonCtrl* pc;
	pc = (CSpinButtonCtrl*)GetDlgItem(IDC_MAXSPEEDSPIN);
	pc->SetPos(0);
	
    // set mode to interactive, disabling filename window for batch
	p = (CButton*)GetDlgItem(IDC_INTERACTIVEMODERADIO);
	p->SetCheck(BST_CHECKED);
	p = (CButton*)GetDlgItem(IDC_BATCHMODERADIO);
	p->SetCheck(BST_UNCHECKED);
	qID = (CEdit*)GetDlgItem(IDC_BATCHFILEEDIT);
	qID->SetWindowText("");
	qID->EnableWindow(FALSE);
	Mode = INTERACTIVE;

	// set current selection of PhotoCell passed combo to 0
	CComboBox * c = (CComboBox*)GetDlgItem(IDC_PCPASSEDCOMBO);
	c->SetCurSel(-1);   // the -1 specifies a clear editbox

    // clear contents of NextPC, disables window until PCPassed is picked
	CComboBox * npc = (CComboBox*)GetDlgItem(IDC_NEXTPCCOMBO);
	npc->ResetContent();   
	npc->EnableWindow(FALSE);

 // clear contents of Turnout, disables window until PCPassed is picked
	CComboBox * toc = (CComboBox*)GetDlgItem(IDC_TURNOUTCOMBO);
	toc->ResetContent();
	toc->EnableWindow(FALSE);

	c = (CComboBox*)GetDlgItem(IDC_TRAINDIRCOMBO);
	c->SetCurSel(-1);  // shows nothing, forcing user to pick


	c = (CComboBox*)GetDlgItem(IDC_TRACKDIRCOMBO);
	c->EnableWindow(FALSE);   // shows nothing, disabled until PCPassed picked


}
// called by constructor when dialog window being created ...
// specifies initial contents, ranges, etc of all controls that
// can be determined at this time, 
// calls SetDefaults for values that must be reset on clear or add
BOOL CAddTrainDialog::OnInitDialog() 
{ 
	CDialog::OnInitDialog();
 
	ifstream fin;
	char Line[80];

	fin.open("id.txt");
	if (!fin.fail())
	{
		CComboBox* q = (CComboBox*)GetDlgItem(IDC_EXISTINGTRAINLIST);
		fin.getline(Line, 80);
		while (!fin.eof()) {
			q->AddString(Line);
			fin.getline(Line, 80);
		}	
	}
	fin.close();
 
	CSpinButtonCtrl* pc = (CSpinButtonCtrl*)GetDlgItem(IDC_MAXSPEEDSPIN);
	pc->SetRange(0,MAXSPEED);

	char temp[10];
	CComboBox * c = (CComboBox*)GetDlgItem(IDC_PCPASSEDCOMBO);
	for (int i = 0; i<MAXPHOTOCELLS; i++) 
	{
		c->AddString(itoa(i, temp, 10));
	}   // rather than store all, should this be made to have only unallocated?
	
	c = (CComboBox*)GetDlgItem(IDC_TRAINDIRCOMBO);
	c->AddString("Forward");
	c->AddString("Reverse");
	
	c = (CComboBox*)GetDlgItem(IDC_TRACKDIRCOMBO);
	
	c->InsertString(COUNTERCLOCKWISE,"Counterclockwise");
	c->InsertString(CLOCKWISE, "Clockwise");

	SetDefaults();

    return true;
}

BEGIN_MESSAGE_MAP(CAddTrainDialog, CDialog)

	ON_BN_CLICKED(IDC_ONADD, OnOK)
	ON_BN_CLICKED (IDC_NEWTRAIN, OnRadioButtonClickedNE)
	ON_BN_CLICKED (IDC_EXISTINGTRAIN, OnRadioButtonClickedNE)          
	ON_BN_CLICKED (IDC_BATCHMODERADIO, OnRadioButtonClickedMode)
	ON_BN_CLICKED (IDC_INTERACTIVEMODERADIO, OnRadioButtonClickedMode)
	ON_CBN_SELCHANGE(IDC_EXISTINGTRAINLIST, OnExistingTrainChange)

	ON_CBN_SELCHANGE(IDC_PCPASSEDCOMBO, OnPCPassedSelected)
	ON_CBN_SELCHANGE(IDC_NEXTPCCOMBO, OnNextPCSelected)
	ON_CBN_SELCHANGE(IDC_TRACKDIRCOMBO, OnTrackDirSelected)
	ON_CBN_SELCHANGE(IDC_TURNOUTCOMBO, OnTurnoutSelected)

	ON_COMMAND(IDC_ONCLEAR, OnClear)
	ON_COMMAND(IDC_ONCLOSE, OnCancel)


END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAddTrainDialog message handlers
/////////////////////////////////////////////////////////////////////////////
/****************************************************************************
  OnOK - invoked when ADD! button clicked
       - UpdateData invokes DoDataExchange to get info from controls to object
	   - copies data from object to TRAINPROP which gets passed as a parameter
	      with message sent to parent window (CUserScreen)  (message passed
		        to CUserScreen::OnApply)
	   - calls SetDefaults -- should it?  check what happens if all data not 
	      filled in
*****************************************************************************/
void CAddTrainDialog::OnOK ()
{
	UpdateData (TRUE);    // makes sure DoDataExchange gets called to pass
                          // info from dialog boxes, buttons, to data members of class
	
	TRAINPROP tp;
	tp.MaxSpeed = MaxSpeed;
	tp.TrainID = TrainID;
	strcpy(tp.TrainName, TrainName);
	tp.Mode = Mode;
	tp.Direction = Direction;
	tp.DirAroundTrack = DirAroundTrack;
	tp.PCHas = PCHas;
	tp.NextPC = NextPC;
	tp.TOHas = TOHas;
	strcpy(tp.FileName,FileName);

	SetDefaults();
	ptrCV->SendMessage(WM_USER_APPLY, 0, (LPARAM)&tp);
}

/****************************************************************************
  OnPCPassedSelected - invoked when selection made in PhotoCell Passed combo box
       - contents of NextPC combobox and Turnout combo box are deleted
	   - based on photocell selected, all possible next photocells
	     are entered into the NextPC combo box, all possible turnouts
		 are entered into the Turnout combobox, AND all indices of possibilities
		 from the TrackInfo matrix are stored in array indexes
	   - combo boxes for TrackDirection, Turnout and NextPC are enabled but cleared
	 
*****************************************************************************/
void CAddTrainDialog::OnPCPassedSelected()
{
	CComboBox * c = (CComboBox*)GetDlgItem(IDC_PCPASSEDCOMBO);
	CString SelectedPC;
	c->GetLBText(c->GetCurSel(), SelectedPC);

    // clear contents of Next PC combo, then add those next to PC 0
	CComboBox * npc = (CComboBox*)GetDlgItem(IDC_NEXTPCCOMBO);
	npc->ResetContent();
	CComboBox * toc = (CComboBox*)GetDlgItem(IDC_TURNOUTCOMBO);
	toc->ResetContent();
	char temp[10];

	int j = 0;
	for (int i = 0; i < MAX_TRACKINFO; i++)
	{
		if (!strcmp(itoa(TrackInfo[i][0], temp, 10), SelectedPC) )
		{
			if (CB_ERR == npc->FindStringExact(-1, itoa(TrackInfo[i][1], temp, 10)) )
				npc->AddString(itoa(TrackInfo[i][1], temp, 10));
			if (CB_ERR == toc->FindStringExact(-1, itoa(TrackInfo[i][3], temp, 10)) )
				toc->AddString(itoa(TrackInfo[i][3], temp, 10));
			indexes[j] = i;
			j++;
		}
	}

    CComboBox * trackdir = (CComboBox*)GetDlgItem(IDC_TRACKDIRCOMBO);
	trackdir->EnableWindow(TRUE);
	trackdir->SetCurSel(-1);

	npc->SetCurSel(-1);
	npc->EnableWindow(TRUE);

	toc->EnableWindow(TRUE);
	toc->SetCurSel(-1);
}
/****************************************************************************
  OnNextPCSelected - invoked when selection made in NextPhotoCell combo box
       - PreCondition Passed PhotoCell selected
	   - Knowing Passed PC and NextPC fully determines Turnout and TrackDirection,
	     so correct turnout and track direction are displayed in appropriate
		 combo boxes

*****************************************************************************/
void CAddTrainDialog::OnNextPCSelected()
{
	CComboBox * npc = (CComboBox*)GetDlgItem(IDC_NEXTPCCOMBO);
	int j = npc->GetCurSel();
	if (j == CB_ERR)
		return;  // none selected
	char temp[10];

	CComboBox * toc = (CComboBox*)GetDlgItem(IDC_TURNOUTCOMBO);
	int pos = toc->FindStringExact(-1, itoa(TrackInfo[ indexes[j] ][3], temp, 10) );
	toc->SetCurSel(pos);  // turnout

	CComboBox * trackdir = (CComboBox*)GetDlgItem(IDC_TRACKDIRCOMBO);
	trackdir->SetCurSel(TrackInfo[ indexes[j] ][2]);

}
/****************************************************************************
  OnTrackDirSelected - invoked when selection made in TrackDirection combo box
       - PreCondition Passed PhotoCell selected
	   - PreCondition: Track Direction (counterclockwise or clockwise) selected
	   - Knowing Passed PC and TrackDirection does NOT fully determine 
	     NextPC but DOES fully determine Turnout
	   - NextPC combobox contents are deleted and then updated with only
	     photocells adjacent to PassedPC in the correct direction.  If there
		 is only one, it is selected.
		 
*****************************************************************************/
void CAddTrainDialog::OnTrackDirSelected()
{
	CComboBox * c = (CComboBox*)GetDlgItem(IDC_PCPASSEDCOMBO);
	CString SelectedPC;
	c->GetLBText(c->GetCurSel(), SelectedPC);

	CComboBox * trackdir = (CComboBox*)GetDlgItem(IDC_TRACKDIRCOMBO);
	CString SelectedTrackDir;
	int dir;
	trackdir->GetLBText(trackdir->GetCurSel(), SelectedTrackDir);
	if (!strcmp(SelectedTrackDir, "Clockwise"))
		dir = CW;
	else if (!strcmp(SelectedTrackDir, "Counterclockwise"))
		dir = CC;
    else
		return;  // no selection made

	CComboBox * npc = (CComboBox*)GetDlgItem(IDC_NEXTPCCOMBO);
	npc->ResetContent();
	char temp[10];

	CComboBox * toc = (CComboBox*)GetDlgItem(IDC_TURNOUTCOMBO);
    toc->ResetContent();
	// could be more than one NextPC or TO for chosen track direction
	int k = 0;
	for (int i = 0; i < MAX_TRACKINFO; i++)
	{
		if (!strcmp(itoa(TrackInfo[i][0], temp, 10), SelectedPC) 
			&& (dir == TrackInfo[i][2]) )
		{
			if (CB_ERR == npc->FindStringExact(-1, itoa(TrackInfo[i][1], temp, 10)) )
				npc->AddString(itoa(TrackInfo[i][1], temp, 10));
			if (CB_ERR == toc->FindStringExact(-1, itoa(TrackInfo[i][3], temp, 10)) )
				toc->AddString(itoa(TrackInfo[i][3], temp, 10));
			indexes[k] = i;
			k++;
		}
	}
	if (toc->GetCount() == 1)
		toc->SetCurSel(0);
	if (npc->GetCount() == 1)
		npc->SetCurSel(0);
	else
		npc->SetCurSel(-1);

}

/****************************************************************************
  OnTurnoutSelected - invoked when selection made in Turnout combo box
       - PreCondition Passed PhotoCell selected
	   - PreCondition: Turnout selected  
	   - Knowing Passed PC and Turnout does NOT fully determine NextPC but
	     DOES determine TrackDirection
	   - NextPC combobox contents are deleted and then updated with only
	     photocells reachable from PassedPC via selected Turnout.  If there
		 is only one, it is selected.
		 
*****************************************************************************/
void CAddTrainDialog::OnTurnoutSelected()
{
    CComboBox * c = (CComboBox*)GetDlgItem(IDC_PCPASSEDCOMBO);
	CString SelectedPC;
	c->GetLBText(c->GetCurSel(), SelectedPC);

	CComboBox * toc = (CComboBox*)GetDlgItem(IDC_TURNOUTCOMBO);
	CString SelectedTurnout;
	toc->GetLBText(toc->GetCurSel(), SelectedTurnout);

	CComboBox * npc = (CComboBox*)GetDlgItem(IDC_NEXTPCCOMBO);
	npc->ResetContent();
	char temp[10];

	CComboBox * trackdir = (CComboBox*)GetDlgItem(IDC_TRACKDIRCOMBO);
	// could be more than one NextPC for chosen turnout
	int k = 0;
	int dir;
	for (int i = 0; i < MAX_TRACKINFO; i++)
	{
		if (!strcmp(itoa(TrackInfo[i][0], temp, 10), SelectedPC) 
			&& (!strcmp(itoa(TrackInfo[i][3], temp, 10), SelectedTurnout ) )  )
		{
			if (CB_ERR == npc->FindStringExact(-1, itoa(TrackInfo[i][1], temp, 10)) )
				npc->AddString(itoa(TrackInfo[i][1], temp, 10));
			dir = TrackInfo[i][2];
			indexes[k] = i;
			k++;
		}
	}
	
	trackdir->SetCurSel(dir);
	if (npc->GetCount() == 1)
		npc->SetCurSel(0);
	else
		npc->SetCurSel(-1);


}
/****************************************************************************
  OnRadioButtonClickedNE - invoked whenever New or Existing button clicked
      - if existing, combobox enabled so user can make selection, edit
	     boxes for ID and Name disabled.
	  - if new, combobox disabled and ID and Name edit boxes enabled
		 
*****************************************************************************/
void CAddTrainDialog::OnRadioButtonClickedNE()
{
	//get handle on listbox
	CComboBox* p = (CComboBox*)GetDlgItem(IDC_EXISTINGTRAINLIST);
	CEdit* qID = (CEdit*)GetDlgItem(IDC_TRAINIDEDIT);
	CEdit* qName = (CEdit*)GetDlgItem(IDC_TRAINNAMEEDIT);
	char temp[10];
	// get id of checked radio button
    int nid = GetCheckedRadioButton(IDC_NEWTRAIN, IDC_EXISTINGTRAIN);

	switch (nid){
		case IDC_NEWTRAIN:		p->EnableWindow(FALSE);			
								qID->EnableWindow(TRUE);
								qName->EnableWindow(TRUE);
								break;

		case IDC_EXISTINGTRAIN:	p->EnableWindow(TRUE);
								p->GetLBText(p->GetCurSel(), temp);
								qID->SetWindowText(temp);
								qID->EnableWindow(FALSE);
								qName->EnableWindow(FALSE);
								qName->SetWindowText ("hey!");
								break;
	}//end switch
}

/****************************************************************************
  OnRadioButtonClickedMode - invoked whenever Batch or Interactive button clicked
      - if Interactive, filename edit box disabled
	  - if Batch, filename edit box enabled
*****************************************************************************/
void CAddTrainDialog::OnRadioButtonClickedMode()
{
	//get handle on listbox
	CEdit* pBatch = (CEdit*)GetDlgItem(IDC_BATCHFILEEDIT);
	CButton* pInR = (CButton*)GetDlgItem(IDC_INTERACTIVEMODERADIO);
	CButton* pBaR = (CButton*)GetDlgItem(IDC_BATCHMODERADIO);


	// get id of checked radio button
    int nid = GetCheckedRadioButton(IDC_BATCHMODERADIO, IDC_INTERACTIVEMODERADIO);

	switch (nid){
		case IDC_BATCHMODERADIO:	
										Mode = BATCH;
										pBatch->EnableWindow(TRUE);
										break;

		case IDC_INTERACTIVEMODERADIO:  
										Mode = INTERACTIVE;
										pBatch->EnableWindow(FALSE);
										break;
	}//end switch
}


/****************************************************************************
  OnExistingTrainChange - invoked when selection is made in combobox of 
                          existing trains
      - updates ID and Name edit boxes appropriately
*****************************************************************************/
void CAddTrainDialog::OnExistingTrainChange()
{	CComboBox* p = (CComboBox*)GetDlgItem(IDC_EXISTINGTRAINLIST);
	CEdit* qID = (CEdit*)GetDlgItem(IDC_TRAINIDEDIT);
	CEdit* qName = (CEdit*)GetDlgItem(IDC_TRAINNAMEEDIT);
	char temp[10];

	p->GetLBText(p->GetCurSel(), temp);
	qID->SetWindowText(temp);
	qName->SetWindowText ("hey!");

}

/****************************************************************************
  OnClear - invoked when Clear button is clicked
          - calls SetDefaults to restore default values
*****************************************************************************/
void CAddTrainDialog::OnClear ()
{
	SetDefaults();
}

/****************************************************************************
  OnCancel - invoked when Close button is clicked or cancel button 
              (upper right X) is clicked
          - calls DestroyWindow to delete window
		  - MUST override OnCancel for modeless dialogs since cannot
		    allow default OnCancel to call EndDialog
*****************************************************************************/
void CAddTrainDialog::OnCancel ()
{
//	MessageBox("OnCancel", "CAddTrainDialog");
	DestroyWindow ();
}

/****************************************************************************
  PostNcDestroy - invoked by DestroyWindow (from OnCancel) 
          - calls base class CDialog::PostNCDestroy
		  - sends message to parent window that it is being destroyed
		     (necessary to parent can make pointer NULL)
		  - deallocates memory for self with delete this;
*****************************************************************************/
void CAddTrainDialog::PostNcDestroy () 
{	
//	MessageBox("PostNcDestroy", "CAddTrainDialog");

	CDialog::PostNcDestroy ();
	ptrCV->SendMessage(WM_USER_DIALOG_DESTROYED, 0, 0);
    delete this;
}