#if !defined TRAIN_H

#define TRAIN_H

#include <string>
#include "MyDcc.h"
using namespace std;

const int BATCH = 0;
const int INTERACTIVE = 1;

const int CLOCKWISE = 1;
const int COUNTERCLOCKWISE = 0;
const int FORWARD = 1;
const int REVERSE = 0;

const int ON = 1;
const int OFF = 0;

const int SENTCOMMAND = 1;
const int QUEUECOMMAND = 0;
const int MAXINPUTLINES = 100;


class Train
{
public:

	Train(int Id1, char Name[], int PhotocellHas1, int PhotocellNext1, int TurnoutHas1, 
		  int Mode1, char Filename1[] = "", int MaxSpeed1 = 16, 
		  int DirectionAroundTrack1 = CLOCKWISE, int Direction1 = FORWARD);

/////////////////////////////////////////////////////////////////////////////
//Constructs a Train-- Needs an id, name, the photocell that the Train Has to start off
//with (ie the one that it already passed) The photocell that it will hit next
//The turnout that it has.  
//Defaulted mode = INTERACTIVE, MaxSpeed = 16, DirectionAroundTrack = ClockWise, 
//Direction = Forward
///////////////////////////////////////////////////////////////////////////////

	Train( const Train & T);
//////////////////////////////////////////////////////////////////////
//Copys a Train
///////////////////////////////////////////////////////////////////

	virtual ~Train();

	int GetId();
//////////////////////////////////////////////////////////////////////
//Returns the id of a train
////////////////////////////////////////////////////////////////////

	int GetSpeed();
//////////////////////////////////////////////////////////////////////
//returns the speed of a train
////////////////////////////////////////////////////////////////////	

	int GetPhotocellHas();
///////////////////////////////////////////////////////////////////////
//Returns the PhotoCell the Train Has(ie the photocell it already covered
//bounding the track that it is on
/////////////////////////////////////////////////////////////////////////

	int GetLastPhotocellCovered();
/////////////////////////////////////////////////////////////////////////
//Returns the last photocell covered
/////////////////////////////////////////////////////////////////////////

	int GetPhotocellNext();
///////////////////////////////////////////////////////////////////////
//Returns the PhotoCell the Train will have next(ie the photocell it did not covered
//bounding the track that it is on
////////////////////////////////////////////////////////////////////

	int GetTurnoutHas();
///////////////////////////////////////////////////////////////////////
//Returns the Turnout the Train Has(ie the Turnout it has on the track that it is on
/////////////////////////////////////////////////////////////////////////


	int GetBlockingPhotocell();
/////////////////////////////////////////////////////////////////
//Returns Photocell that is blocking the Train
////////////////////////////////////////////////////////////////

	int GetDirectionAroundTrack();
//////////////////////////////////////////////////////////////////////////
//Returns the Train's DirectionAroundTrack ie (Clockwise or Counterclockwise
////////////////////////////////////////////////////////////////////////////

	int GetMaxSpeed();
////////////////////////////////////////////////
//Returns Trains MaxSpeed
/////////////////////////////////////////////////

	int GetDirection();
////////////////////////////////////////////////////////////////////////////
//Returns the Trains Direction ie Forward or Reverse
////////////////////////////////////////////////////////////////////////////

	int GetHeadLight();	
//////////////////////////////////////////////////////////
//Return ON or OFF for current State of Headlight
///////////////////////////////////////////////////////////

	int GetTailLight();
//////////////////////////////////////////////////////////
//Return ON or OFF for current State of Taillight
///////////////////////////////////////////////////////////

	int GetMode();
////////////////////////////////////////////////////////////////////////////
//Returns the Mode ie INTERACTIVE OR BATCH of the train
///////////////////////////////////////////////////////////////////////////

	int GetBlockedStatus();
//////////////////////////////////////////////////
//Returns Active or Blocked for Status of Train
//////////////////////////////////////////////////

	char * GetName();
/////////////////////////////////////////////////////////
//Returns name of Train
////////////////////////////////////////////////////////

	Train * link;  //link for linked lists

	friend class ResourceManager;  //Resource Manager can adjust private variables

private:


	int SetSpeed(int newspeed);
/////////////////////////////////////////////////////////////////////
//Sets the Speed of a train to be specified speed Calls Increase Speed
//or DecreaseSpeed to adjust speed as necessary returns the value of 
//IncreaseSpeed or DecreaseSpeed return 1 if speed did not change
////////////////////////////////////////////////////////////////////////
	int Stop();
////////////////////////////////////////////////////////////////
//Stops a train by setting its speed to 0
///////////////////////////////////////////////////////////////
	int ChangeDirection();
///////////////////////////////////////////////////////////////////////////////
//Changes the Direction of a train.  Remembers the oldSpeed Stops the Train toggles 
//the DirectionAroundTrack. Toggles the Train's direction
//Swaps the PhotocellHas and PhotoCellNext and starts the train again with 
//remembered old speed
//////////////////////////////////////////////////////////////////////////////////
	int TurnHeadLightOn(int on);
/////////////////////////////////////////////////////////////////////////
//Toggles the HeadLight if inputed Parameter does not match the current state
//of the Headlight  Sends fuction command to systemone box
//////////////////////////////////////////////////////////////////////////
	int ToggleHeadLight();
///////////////////////////////////////////////////////////////////
//Toggles the Current State of the HeadLight and send the command to the
//systemOne board
//////////////////////////////////////////////////////////////////////
	int TurnTailLightOn(int on);
/////////////////////////////////////////////////////////////////////////
//Toggles the TailLight if inputed Parameter does not match the current state
//of the Headlight send function command to systemone box
//////////////////////////////////////////////////////////////////////////
	int ToggleTailLight();
///////////////////////////////////////////////////////////////////
//Toggles the Current State of the TailLight and send the command to the
//systemOne board
//////////////////////////////////////////////////////////////////////
	int ReadInstruction(int &pc, int& to, string& command);
/////////////////////////////////////////////////////////////////////
//Reads the next photocell for a Train running on batch mode
//note that all files end with 9999 which represents a loop which begins
//at the first photocell again.  This closes, opens and begins reading 
//from the file again
////////////////////////////////////////////////////////////////////////

	int ExecuteLastDCC_Command();
/////////////////////////////////////////////////////
//Sets speed to the last User supplied Speed
//////////////////////////////////////////////////////

	int LastUserSpeed;			//Last speed that the user issued 
	int Id;						//Trains id number
	int Speed;					//Trains current speed
	int MaxSpeed;				//Trains Max Speed
	int OldPhotocellHas;		//The last photocell the train had
	int PhotocellHas;			//The photocell the Train has already passed
	int PhotocellNext;			//The photocell the train will pass
	int TurnoutHas;				//The turnout between pchas and pcnext
	int OldTurnoutHas;			//The last turnout the train had
	int LastPhotocellCovered;	//The Last photocell the train covered
	int DirectionAroundTrack;	//Train direction Clockwise or Counterclockwise
	int Direction;				//Trains direction Forward or reverse
	int HeadLight;				//Headlight Bool---on or off
	int TailLight;				//Taillight Bool --- on or off
	int Mode;					//Trains mode -- Batch or interactive
	int BlockInstructions;		//Blocked Commands--Can't change direction over pc
	int BlockingPhotocell;		//The photocell the train couldn't have
	int BlockingTurnout;
	int BlockedStatus;			//Status: Blocked or Active
	char Name[50];				//Trains name
	char InputFilename[80];		//Input file for batch mode
	string DCCLastCommand;		//Lastcommand--currently not used
	string Input[MAXINPUTLINES];//Input array for batch mode
	int LinesRead;				//Number of lines read on input file for batch mode
	int cursor;					//Current place of reading of Input array

};

void swap(int & x, int & y);
////////////////////////////////////////
//Swaps two integers
///////////////////////////////////////

#endif


