#include <afxwin.h>
#include "stdafx.h"
#include "CTrainControl.h"
#include "ResourceManager.h"
#include "Constants.h"
#include "Train.h"
#include "MyToolTip.h"


/////////////////////////////////////////////////////////////////////////
//Construct a Train Control to allow a user to control a single train
//Sets its pointer to the resource manager it needs to call to send
// train commands and create the window.  Sets timer to update data from 
// Train
//////////////////////////////////////////////////////////////////////
CTrainControl::CTrainControl(Train * T, ResourceManager * ResMng)
{

	char Title[60];

	MyResMng = ResMng;

	MyTrain = T;

	strcpy(Title, "Train Control ");
	strcat(Title, T->GetName());

	CRect Rect(0,0,500, 500);  //define oringal window
	Create(0, Title, WS_OVERLAPPEDWINDOW, Rect, NULL, 0);  //create new window

 	CRect MyRect(0, 0, m_cxChar*45,  m_cyChar * 18); 
	// comes back with m_cxChar and my_cyChar defined set up new rect for new window
										

	CalcWindowRect(&MyRect);  // calc pixel width and height for window

	SetWindowPos(NULL, 0, 0, MyRect.Width() , MyRect.Height(), 
		         SWP_NOZORDER | SWP_NOMOVE | SWP_NOREDRAW);

	SetTimer( 2, 1000, NULL );	// set timer for every second

}

//////////////////////////////////////////////////////////////////
//Kills timer 2
///////////////////////////////////////////////////////////////
CTrainControl::~CTrainControl()
{
	MessageBox("destructor", "CTrainControl");
	if (KillTimer( 2) )
		MessageBox("successfully killed", "the timer");   //kill timer 2
	else
		MessageBox("Couldnt kill", "the timer");
	MessageBox("destructor", "CTrainControl2");
}

///////////////////////////////////////////////////////////////////////
//Creates the display for the window in relative size to chosen font and 
//Creates all buttons, Labels, and Controls for a train
////////////////////////////////////////////////////////////////////
int CTrainControl::OnCreate(LPCREATESTRUCT lpcs)
{

	if (CWnd::OnCreate(lpcs) == -1)
		return -1; //return if not good

	// Used to output messages on the screen and to convert int to ascii
	// to display values of Train object
	char IdMessage[30] = "Train ID: ";
	char MaxSpeedMessage[30] = "Max Speed: ";
	char ModeMessage[30] = "Mode: ";
	char BlockedStatusMessage[30] = "Status: ";
	char DirectionAroundTrackMessage[30] = "Dir Around Track: ";
	char DirectionMessage[30] = "Direction: ";
	char ToHasMessage[30] = "Allocated: ";
	char PcHasMessage[30] = "Allocated: ";
	char PcNextMessage[30] = "Allocated: ";
	char Temp[30];

	//convert int values to ascii to display on the screen
	itoa(MyTrain->GetId(), Temp, 10);
	strcat(IdMessage, Temp);

	itoa(MyTrain->GetMaxSpeed(), Temp, 10);
	strcat(MaxSpeedMessage, Temp);

	itoa(MyTrain->GetPhotocellHas(), Temp, 10);
	strcat(PcHasMessage, Temp);

	itoa(MyTrain->GetPhotocellNext(), Temp, 10);
	strcat(PcNextMessage, Temp);

	itoa(MyTrain->GetTurnoutHas(), Temp, 10);
	strcat(ToHasMessage, Temp);

	if (MyTrain->GetMode() == BATCH)
		strcat(ModeMessage, "Batch");
	else
		strcat(ModeMessage, "Interactive");

	if (MyTrain->GetBlockedStatus() == ACTIVE)
		strcat(BlockedStatusMessage, "Active");
	else
		strcat(BlockedStatusMessage, "Blocked");

	if (MyTrain->GetDirectionAroundTrack() == CLOCKWISE)
		strcat(DirectionAroundTrackMessage, "CW");
	else
		strcat(DirectionAroundTrackMessage, "CC");

	if (MyTrain->GetDirection() == FORWARD)
		strcat(DirectionMessage, "Forward");
	else
		strcat(DirectionMessage, "Reverse");

	m_font.CreatePointFont(65, _T("MS Sans Serif"));  //select our font
	CClientDC dc (this);  //get handler to compute text averages
	CFont * pOldFont = dc.SelectObject(&m_font); // select our font

	TEXTMETRIC Tm; // get Textmetric

	dc.GetTextMetrics(&Tm);  // and give it our font to calculate
	m_cxChar = Tm.tmAveCharWidth;  // store average width
	m_cyChar = Tm.tmHeight + Tm.tmExternalLeading;	// and height

	dc.SelectObject(pOldFont); // return old pen to not consume resources

	CRect rect;
	
	

	// Creates the Diection group box and buttons
	rect.SetRect (m_cxChar * 1, m_cyChar * 5, m_cxChar * 22, m_cyChar * 11);
	DirectionGroupBox.Create (_T ("Direction Information"),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);
	
		rect.SetRect (m_cxChar * 2, (int)(m_cyChar * 6.5), m_cxChar * 20, m_cyChar * 8);
		ChangeDirectionButton.Create (_T ("Change Direction"), WS_CHILD | WS_VISIBLE |
			BS_PUSHBUTTON, rect, this, IDC_CHANGEDIRECTIONBUTTON);


		rect.SetRect (m_cxChar * 2, (int)(m_cyChar * 8.5), m_cxChar * 21, (int)(m_cyChar * 9.5));
		DirectionAroundTrackText.Create (_T (DirectionAroundTrackMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
		         this, IDC_DIRAROUNDTRACKTEXT);

		rect.SetRect (m_cxChar * 2, (int)(m_cyChar * 9.5), m_cxChar * 21, (int)(m_cyChar *10.5));
	    DirectionText.Create (_T (DirectionMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
		         this, IDC_DIRECTIONTEXT);


	//Creates the Speed group box, brake button and Spin controler for speed
	rect.SetRect (m_cxChar * 24, m_cyChar * 0, m_cxChar * 44, m_cyChar * 4);
	TrainSpeedGroupBox.Create (_T (MaxSpeedMessage),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);

		rect.SetRect (m_cxChar * 25, (int)(m_cyChar * 1.5), m_cxChar * 32, m_cyChar * 3);
		StopButton.Create (_T ("Brake"), WS_CHILD | WS_VISIBLE |
			BS_PUSHBUTTON, rect, this, IDC_STOPBUTTON);

		rect.SetRect (m_cxChar * 33, (int)(m_cyChar * 1.5), m_cxChar * 37, m_cyChar * 3);
		SpeedEditBox.Create (WS_CHILD|WS_VISIBLE|WS_BORDER, rect, this, 
				IDC_SPEEDEDIT);

		rect.SetRect (m_cxChar * 37, (int)(m_cyChar * 1.5), m_cxChar * 39, m_cyChar * 3);
		SpeedControl.Create (UDS_ARROWKEYS | UDS_SETBUDDYINT | WS_VISIBLE | UDS_AUTOBUDDY  , rect, 
			    this, IDC_SPEEDCONTROL);		


		//Creates the light group box with toggle button for tail/headlights
    rect.SetRect (m_cxChar * 24, m_cyChar * 5, m_cxChar * 44, m_cyChar * 11);
    LightsGroupBox.Create (_T ("Light Information"),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);

		rect.SetRect (m_cxChar * 27, (int)(m_cyChar * 6.5), m_cxChar * 43, m_cyChar * 8);
		HeadLightToggleButton.Create (_T ("HeadLight Off"), WS_CHILD | WS_VISIBLE |
			 BS_PUSHBUTTON, rect, this, IDC_HEADLIGHTTOGGLEBUTTON);

		rect.SetRect (m_cxChar * 27, (int)(m_cyChar * 8.5), m_cxChar * 43, m_cyChar * 10);
		TailLightToggleButton.Create (_T ("TailLight Off"), WS_CHILD | WS_VISIBLE |
			 BS_PUSHBUTTON, rect, this, IDC_TAILLIGHTTOGGLEBUTTON);



		//Creates General info id, blocked status text output 
	rect.SetRect (m_cxChar * 1, m_cyChar * 0, m_cxChar * 22, m_cyChar * 4);
    TrainInfoGroupBox.Create (_T (IdMessage),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);


		rect.SetRect (m_cxChar * 2, m_cyChar * 1, m_cxChar * 21, m_cyChar * 2);
		ModeText.Create (_T (ModeMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
		         this, IDC_MODETEXT);

		rect.SetRect (m_cxChar * 2, m_cyChar * 2, m_cxChar * 21, m_cyChar * 3);
		BlockedStatusText.Create (_T (BlockedStatusMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
			this, IDC_BATCHSTATUSTEXT);

		//Creates the allocations windows for photocells
	rect.SetRect (m_cxChar * 1, m_cyChar * 12, m_cxChar * 22, m_cyChar * 16);
    PhotocellResourceGroupBox.Create (_T ("Train Photocells"),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);

	     rect.SetRect (m_cxChar * 2, m_cyChar * 13, m_cxChar * 21, m_cyChar * 14);
		 PhotocellhasText.Create (_T (PcHasMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
			this, IDC_PCNEXTTEXT);

		 rect.SetRect (m_cxChar * 2, m_cyChar * 14, m_cxChar * 21, m_cyChar * 15);
		 PhotocellnextText.Create (_T (PcNextMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
			this, IDC_PCHASTEXT);


		 //Create the allocation area for turnouts
	rect.SetRect (m_cxChar * 24, m_cyChar * 12, m_cxChar * 44, m_cyChar * 16);
	TurnoutResourceGroupBox.Create (_T ("Train Turnouts"),  WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
        rect, this, (UINT) -1);

		rect.SetRect (m_cxChar * 25, m_cyChar * 13, m_cxChar * 43, m_cyChar * 14);
		TurnoutHas.Create (_T (ToHasMessage), WS_CHILD | WS_VISIBLE | SS_LEFT, rect,
			this, IDC_TOHASTEXT);


		//Creates the tool tips for the buttons
	m_ctlTTrainControl.Create (this);
    m_ctlTTrainControl.AddWindowTool (GetDlgItem (IDC_STOPBUTTON),
        MAKEINTRESOURCE (IDS_STOPBUTTON));

	 m_ctlTTrainControl.AddWindowTool (GetDlgItem (IDC_CHANGEDIRECTIONBUTTON),
        MAKEINTRESOURCE (IDS_CHANGEDIRECTIONBUTTON));

	 m_ctlTTrainControl.AddWindowTool (GetDlgItem (IDC_HEADLIGHTTOGGLEBUTTON),
        MAKEINTRESOURCE (IDS_HEADLIGHTTOGGLEBUTTON));

	 m_ctlTTrainControl.AddWindowTool (GetDlgItem (IDC_TAILLIGHTTOGGLEBUTTON),
        MAKEINTRESOURCE (IDS_TAILLIGHTTOGGLEBUTTON));

	 m_ctlTTrainControl.AddWindowTool (GetDlgItem (IDC_SPEEDCONTROL),
        MAKEINTRESOURCE (IDS_SPEEDCONTROL));

	 m_ctlTTrainControl.AddWindowTool (GetDlgItem (IDC_TOHASTEXT),
        MAKEINTRESOURCE (IDS_TOHASTEXT));

	 //Set fonts to all controls we made
	ChangeDirectionButton.SetFont (&m_font, FALSE);
	StopButton.SetFont (&m_font, FALSE);
	TailLightToggleButton.SetFont (&m_font, FALSE);
	HeadLightToggleButton.SetFont (&m_font, FALSE);
	ModeText.SetFont (&m_font, FALSE);
	BlockedStatusText.SetFont (&m_font, FALSE);
	TrainInfoGroupBox.SetFont (&m_font, FALSE);
	TrainSpeedGroupBox.SetFont (&m_font, FALSE);
	TurnoutHas.SetFont (&m_font, FALSE);
	PhotocellnextText.SetFont (&m_font, FALSE);
	PhotocellhasText.SetFont (&m_font, FALSE);
	LightsGroupBox.SetFont (&m_font, FALSE);
	DirectionGroupBox.SetFont (&m_font, FALSE);
	SpeedEditBox.SetFont (&m_font, FALSE);
	DirectionAroundTrackText.SetFont (&m_font, FALSE);
	DirectionText.SetFont (&m_font, FALSE);
	TurnoutResourceGroupBox.SetFont (&m_font, FALSE);
	PhotocellResourceGroupBox.SetFont (&m_font, FALSE);
    WindowBackgroundColor = ::GetSysColor (COLOR_3DFACE);

	//Set speed spinners range
	SpeedControl.SetRange(0, MyTrain->GetMaxSpeed());


	return 0;


}//Oncreate

////////////////////////////////////////////////////////////////////
//repaint the rect gray
////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnPaint()
{


  CRect rect;
  GetClientRect(&rect);
  CPaintDC dc(this);
  dc.FillSolidRect(rect, WindowBackgroundColor);
  //just fill the rect gray
   
 
}//OnPaint


/////////////////////////////////////////////////////////////////////////
//Called when timer fires.  Updates current train status by calling get methods
//in train
// updates allocation status, photocell and turnout, speed, blocked status, and
//headlight/taillight
////////////////////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnTimer()
{

	char temp[10];

	//Used to output messages to user
	char BlockedStatusMessage[30] = "Status: ";
	char ToHasMessage[30] = "Allocated: ";
	char PcHasMessage[30] = "Allocated: ";
	char PcNextMessage[30] = "Allocated: ";

	//convert ints to ascii to ouput on scrren
	itoa(MyTrain->GetSpeed(), temp, 10);
	SpeedEditBox.SetWindowText(temp);

	itoa(MyTrain->GetPhotocellHas(), temp, 10);
	PhotocellhasText.SetWindowText(strcat(PcHasMessage, temp));

	itoa(MyTrain->GetPhotocellNext(), temp, 10);
	PhotocellnextText.SetWindowText(strcat(PcNextMessage, temp));

	//get current turnout allocation and put it out on screen
	if (MyTrain->GetTurnoutHas() != -1)
	{
		itoa(MyTrain->GetTurnoutHas(), temp, 10);
		TurnoutHas.SetWindowText(strcat(ToHasMessage, temp));
	}
	else
		TurnoutHas.SetWindowText("Allocated: None");


	//get current status blocked or active and output it on screen
	if (MyTrain->GetBlockedStatus() == ACTIVE)
		BlockedStatusText.SetWindowText(strcat(BlockedStatusMessage, "Active"));
	else
		BlockedStatusText.SetWindowText(strcat(BlockedStatusMessage, "Blocked"));


	//get current headlight status and output on or off on screen
	if (MyTrain->GetHeadLight() == ON)
		HeadLightToggleButton.SetWindowText("HeadLight On");
	else
		HeadLightToggleButton.SetWindowText("HeadLight Off");

	//Get current taillight status and output on or off on screen
	if (MyTrain->GetTailLight() == ON)
		TailLightToggleButton.SetWindowText("TailLight On");
	else
		TailLightToggleButton.SetWindowText("TailLight Off");


}


/////////////////////////////////////////////////////////////////
//Traps user clicked Brake button calls resourcemanager method to 
//make train stop
////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnStop()
{

	
	MyResMng->Stop(MyTrain->GetId());
	char temp[10];
	itoa(MyTrain->GetSpeed(), temp, 10);
	SpeedEditBox.SetWindowText(temp);


}

/////////////////////////////////////////////////////////////////
//Traps user pressing ChangeDirection and calls resource manager method
// to change the direction of a train
/////////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnChangeDirection()
{

	MyResMng->ChangeDirection(MyTrain->GetId());
	if (MyTrain->GetDirection() == FORWARD)
		DirectionText.SetWindowText("Direction: Forward");
	else
		DirectionText.SetWindowText("Direction: Reverse");

	if (MyTrain->GetDirectionAroundTrack() == CLOCKWISE)
		DirectionAroundTrackText.SetWindowText("Dir Around Track: CW");
	else
		DirectionAroundTrackText.SetWindowText("Dir Around Track: CC");

}

/////////////////////////////////////////////////////////////////////////
//Traps user pressing headlight button and calls resource manager to toggle
//headlight of train
////////////////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnHeadLightToggle()
{


	MyResMng->ToggleHeadLight(MyTrain->GetId());

	if (MyTrain->GetHeadLight() == ON)
	{
			HeadLightToggleButton.SetWindowText("HeadLight On");
	}
	else
	{
			HeadLightToggleButton.SetWindowText("HeadLight Off");
	}
		
	


}

////////////////////////////////////////////////////////////////////////////
//Traps scrolling inside of speed control and uses resource manager to set the 
//speed of a train
/////////////////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnVScroll (UINT nCode, UINT nPos, CScrollBar* pScrollBar)
{

	int speed;
	char speedchar[5];
	SpeedEditBox.GetWindowText(speedchar,5);
	speed = atoi(speedchar);

	MyResMng->SetSpeed(MyTrain->GetId(), speed);

}


/////////////////////////////////////////////////////////////////////////
//Traps user pressing taillight button and calls resource manager to toggle
//taillight of train
////////////////////////////////////////////////////////////////////////////
afx_msg void CTrainControl::OnTailLightToggle()
{

	MyResMng->ToggleTailLight(MyTrain->GetId());

	if (MyTrain->GetTailLight() == ON)
	{
	
			TailLightToggleButton.SetWindowText("TailLight On");
	
	}
	else
	{
			TailLightToggleButton.SetWindowText("TailLight Off");

	}
}


BEGIN_MESSAGE_MAP(CTrainControl, CFrameWnd)

	
	ON_WM_CREATE()

	ON_WM_TIMER()   
	
	ON_WM_PAINT()   

	ON_COMMAND(IDC_STOPBUTTON, OnStop)

	ON_COMMAND(IDC_CHANGEDIRECTIONBUTTON, OnChangeDirection)

	ON_COMMAND(IDC_HEADLIGHTTOGGLEBUTTON, OnHeadLightToggle)

	ON_COMMAND(IDC_TAILLIGHTTOGGLEBUTTON, OnTailLightToggle)

	ON_WM_VSCROLL ()
	
END_MESSAGE_MAP()


void CTrainControl::PostNcDestroy()
{
MessageBox("Ctraincontrol", "postncdestroy");

	int ID = MyTrain->GetId();
	AfxGetMainWnd()->SendMessage(WM_USER_TRAINCONTROL_DESTROYED, 0, (LPARAM)&ID);
	MessageBox("Ctraincontrol", "postncdestroy - after CUserScreen::OnTrainDestroyed");
    delete this;
	
}