#if !defined CUSERSCREEN_H

#define CUSERSCREEN_H

#include "ResourceManager.h"
#include "CTrainControl.h"
#include "CAddTrainDialog.h"
#include "Turnout.h"
#include "Photocells.h"
#include "PhotocellTextBoxes.h"
#include "MyToolTip.h"


const int UpperLeftScreenX = 0;
const int UpperLeftScreenY = 0;
const int SPACER = 50;
const int MAXTRAINS = 10;


class CUserScreen:public CFrameWnd 
{

  public:

	 CUserScreen();
/////////////////////////////////////////////////////////////////////
//Constructor for CUserScreen.  Creates the window which is ajusted in OnCreate
//creates a resourcemanger to handle operating system and creates TrainControls 
//to allow the user to control the trains. Init data structures and sets a timer
//to update visual
/////////////////////////////////////////////////////////////////////////////////

	 ~CUserScreen();
///////////////////////////////////////////////////////////////////
//Deletes the windows I create.  The resourcemanager and the trains pointers
//////////////////////////////////////////////////////////////////////////

	 afx_msg int OnCreate(LPCREATESTRUCT lpcs);
////////////////////////////////////////////////////////////////////////
//Creates layout of screen in proportion to the chosen font.  Creates all 
//Buttons, Labels, ListBoxes and inputs the bitmap. 
//////////////////////////////////////////////////////////////////////

	 afx_msg void OnTimer();
/////////////////////////////////////////////////////////////////////////
//Called when timer is fired.  Cycles through turnouts and photocells to
//check if there was a change since last timer.  If there is we invalide the 
//rect where the change occurred to force a call to OnPaint
///////////////////////////////////////////////////////////////////////////

	 afx_msg void OnExit();
////////////////////////////////////////////////////////////////////////
//Calls when user exits from menu -- send close  SendMessage( WM_CLOSE )
/////////////////////////////////////////////////////////////////////


	 afx_msg void OnDestroy();

///////////////////////////////////////////////////////////////////
//Called when window is destroyed kills the timer
/////////////////////////////////////////////////////////////////

	 afx_msg void OnPaint();
/////////////////////////////////////////////////////////////////////////
//Repainst the screen--Called when user covers or shrinks the window
//Also called by OnTimer to keep the display correct
///////////////////////////////////////////////////////////////////////

	 afx_msg void OnLButtonDown( UINT uFlags, CPoint point );
//////////////////////////////////////////////////////////////////////
//Traps the clicking of Left Mouse Button and checks if the click was
//over a turnout.  If it was it calls the resource manger to toggle the
//turnout if possible
/////////////////////////////////////////////////////////////////////

  private:


	  afx_msg void OnToggleEnabledCourse();
////////////////////////////////////////////////////////////////////////
//Pressed the Enabled/Disabled CourseButton--Calls function with resourcemanger
//to handle it and updates the screen if the resourcemanger was successful
/////////////////////////////////////////////////////////////////////////

	  void LoadTurnouts();
//////////////////////////////////////////////////////////////////////
//Loads the Init Colors and Postions of Turnouts
/////////////////////////////////////////////////////////////////

	  void LoadPhotocells();
///////////////////////////////////////////////////////////////////////////
//Loads the Positon of the photocells
//////////////////////////////////////////////////////////////////////////

	  void LoadPhotocellTextBoxes();
////////////////////////////////////////////////////////////////////////
//Loads the positions of the CStatic Boxes to show ownership of photocells
///////////////////////////////////////////////////////////////////////////

	  CFont m_font;  //font used to set text on user interface

	  //used to create dimensions of screen
	  int m_cxChar;  //avg width of font

	  int m_cyChar;	 //avg height of font 

	  //pointer to the resourcemanager we create in constructor
	  ResourceManager  * MyResourceManager;

	  //Pointers to the Windows we create to let user control trains
	  CTrainControl * TrainControls[MAXTRAINS];
	  //Pointers to All the trains on the course
	  Train * AllTrains[MAXTRAINS];

	  //Number of Actual trains on the course
	  int NumTrains;

	  //Buttons and Listboxes for this window
	  CButton EnableTrackToggleButton;
	  CBitmap redbutton;    // bmcvey
	  CBitmap greenbutton;  // bmcvey
	  CButton CourseControlGroupBox;
	  CListBox UserMessages; 
	  CListBox TrainInfo; 
	  CStatic MessageText;
	  CStatic TrainInfoText;

	  CDC m_memDC; 

	  CBitmap TrackLayoutBitmap; //out track bitmap

	  CPoint UpperLeftBitmap;  //corners of bitmap on screen
	  CPoint LowerRightBitmap;

	  Turnout OutsideTurnouts[MAXTURNOUTS]; 
	  // the outside turnouts closer to the edge
	  Turnout InsideTurnouts[MAXTURNOUTS];
	  // the inside turnouts closer to center
	  Photocells PhotocellPoints[MAXPHOTOCELLS];
	  // the messages stating ownership of photocells
	  PhotocellTextBox PhotocellOwnership[MAXPHOTOCELLS];
	  // Our tooltips
	  CMyToolTipCtrl m_ctlTT;

	  // The Previous and Current States of the Photocells keeps track of
	  // when we need to update the screen because there was a change
	  int PrevPC[MAXPHOTOCELLS];
	  int CurPC[MAXPHOTOCELLS];

	  // The previous and current state of the turnouts used to keep track of 
	  // when we need to update the screen becasue there was a change
	  int PrevTO[MAXTURNOUTS];
	  int CurTO[MAXTURNOUTS];
	  
	  int BitHeight; //bitmaps height

	  int BitWidth; //bitmaps width

  	  COLORREF WindowBackgroundColor; //background color of screen
/***************************************
  added by bmcvey 6-27-01
  facilitates AddTrain dialog -- information passing from dialog to window */
	CAddTrainDialog * myATDlg;
	
	afx_msg void OnAddTrain();
	afx_msg void OnAddTrainFromFile();
	afx_msg LRESULT OnApply (WPARAM wParam, LPARAM lParam);

	afx_msg LRESULT OnDialogDestroyed (WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnTrainDestroyed (WPARAM wParam, LPARAM lParam);
/*****************************************/
	  DECLARE_MESSAGE_MAP()

}; 


int FindMax(int x, int y);
/////////////////////////////////////////////////////////////
//Finds the larger of two integers
////////////////////////////////////////////////////////////


#endif
