#include "train.h"
#include <fstream>
#include <iostream>
using namespace std;

/////////////////////////////////////////////////////////////////////////////
//Constructs a Train-- Needs an id, the photocell that the Train Has to start off
//with (ie the one that it already passed) The photocell that it will hit next
//The turnout that it has.  
//Defaulted mode = INTERACTIVE, MaxSpeed = 16, DirectionAroundTrack = ClockWise, 
//Direction = Forward
///////////////////////////////////////////////////////////////////////////////
Train::Train(int Id1, char Name1 [], int PhotocellHas1, int PhotocellNext1, int TurnoutHas1, 
			 int Mode1, char Filename1[], int MaxSpeed1, int DirectionAroundTrack1, int Direction1)
{
	Id = Id1; // Set id of train
	strcpy(Name, Name1);
	PhotocellHas = PhotocellHas1; // the photocells bounding the track it starts on
	PhotocellNext = PhotocellNext1;
	TurnoutHas = TurnoutHas1; // and the turnout if there is one

	//init rest of trains variables
	DirectionAroundTrack = DirectionAroundTrack1;
	Direction = Direction1;
	Mode = Mode1;
	MaxSpeed = MaxSpeed1;


	BlockingPhotocell = -1;
	OldPhotocellHas = 0;
	BlockInstructions = 1;
	OldTurnoutHas = 0;
	HeadLight = OFF;
	TailLight = OFF;
	strcpy(DCCLastCommand, "");
	Speed = 0;

	ifstream fin;
	int count = 0;
	string aline;

	if (Mode == BATCH && strcmp(Filename, "") != 0)
	{
		fin.open(Filename);
	    getline(fin, InputFile[count]);
		while (!fin.eof())
		{
			count++;
			getline(fin, InputFile[count]);
		

		}


	}


}




Train::~Train()
{

}


////////////////////////////////////////////////////////////////////////
//Copy Constructor Makes a copy of a Train
//////////////////////////////////////////////////////////////////////////
Train::Train(const Train & T){


	 Id = T.Id;
	 Speed = T.Speed;
	 MaxSpeed = T.MaxSpeed;
	 OldPhotocellHas = T.OldPhotocellHas;
	 PhotocellHas = T.PhotocellHas;
	 PhotocellNext = T.PhotocellNext;
	 TurnoutHas = T.TurnoutHas;
	 OldTurnoutHas = T.OldTurnoutHas;
	 LastPhotocellCovered = T.LastPhotocellCovered;
	 DirectionAroundTrack = T.DirectionAroundTrack;
	 Direction = T.Direction;
	 HeadLight = T.HeadLight;
	 TailLight = T.TailLight;
	 Mode = T.Mode;
	 BlockInstructions = T.BlockInstructions;
	 BlockingPhotocell = T.BlockingPhotocell;
	 strcpy(Name, T.Name);
	 strcpy(DCCLastCommand, T.DCCLastCommand);
	 StringArrayCopy(InputFile, T.InputFile, SIZEOFINPUTFILE);

}



/////////////////////////////////////////////////////////////////////
//Sets the Speed of a train to be specified speed Calls Increase Speed
//or DecreaseSpeed to adjust speed as necessary returns the value of 
//IncreaseSpeed or DecreaseSpeed return 1 if speed did not change
////////////////////////////////////////////////////////////////////////
int Train::SetSpeed(int newspeed)
{

	if (newspeed > Speed)
		return (IncreaseSpeed(newspeed - Speed));
	// if new speed greater call IncreaseSpeed with increment between newspeed and
	//current speed

	else if(newspeed < Speed)
		return (DecreaseSpeed(Speed - newspeed));
	// if new speed less call DecreaseSpeed with increment between newspeed and
	//current speed
	else
		return 1;
	
}



////////////////////////////////////////////////////////////////////////////////
//Increase the speed of a train by increment or a default of 1.  Checks to make
//Sure Train does not exceed its MaxSpeed and then sends command to System One
//Box to increaseSpeed through .dll commands
/////////////////////////////////////////////////////////////////////////////////
int Train::IncreaseSpeed(int increment){


   int success;
   int instruction;
   char command[15];
 
  
   if (Speed + increment <= MaxSpeed)
   //make sure not to exceed maxspeed
   {
   
	  for (int i = 0; i<increment; i++){ 
		  // increases speed on an increment of 1

	     instruction = checkdir(Speed + 1 , Direction);
         success = makecommand(QUEUECOMMAND, instruction ,Id, command);
         //command needs to be sent to the port to be executed;
         Speed++; //increase speed in train struct
	  }//end for

	  strcpy(DCCLastCommand, command); //Update last speed command
	 
//	  fout << "Train " << Id << " increased speed to " << Speed <<endl;
      return 1; //increased correctly
   }//end if
   else
   {
//	  fout << "Train " << Id << " couldn't increase speed past maxspeed" << endl;
	  return 0; //cannot increase past maxspeed
   }
}



////////////////////////////////////////////////////////////////////////////////
//Decrease the speed of a train by increment or a default of 1.  Checks to make
//Sure Train does not decrease past 0 and then sends command to System One
//Box to Decrease Speed through .dll commands
/////////////////////////////////////////////////////////////////////////////////
int Train::DecreaseSpeed(int increment)
{


   int success;
   int instruction;
   char command[15];
  
  
   if (Speed - increment >= 0) // make sure not to go past speed zero
   { 
   
	  for (int i = 0; i<increment; i++)
	  {  // decreases speed on an increment of 1
	     instruction = checkdir(Speed - 1 , Direction);
         success = makecommand(QUEUECOMMAND, instruction ,Id, command);
         //command needs to be sent to the port to be executed;        
         Speed--; //decrease speed in train struct
	  }//end for

	  strcpy(DCCLastCommand, command); //increase speed in train struct
	 
//	  fout << "Train " << Id << " Decrease speed to " << Speed << endl;
      return 1; //decreased correctly
   }//end if
   else
   {
//	  fout << "Train " << Id << " Couldn't decrease speed below 0" << endl;
      return 0; //cannot decrease past speed zero
   }
}


////////////////////////////////////////////////////////////////
//Stops a train by setting its speed to 0
///////////////////////////////////////////////////////////////
int Train::Stop()
{
	return (SetSpeed(0));
}



/////////////////////////////////////////////////////////////////////////
//Toggles the HeadLight if inputed Parameter does not match the current state
//of the Headlight  Sends fuction command to systemone box
//////////////////////////////////////////////////////////////////////////
int Train::TurnHeadLightOn(int TurnOn)
{


	if (TurnOn != HeadLight)
		return (ToggleHeadLight());
	else
		return 1;

}

/////////////////////////////////////////////////////////////////////////
//Toggles the TailLight if inputed Parameter does not match the current state
//of the Headlight send function command to systemone box
//////////////////////////////////////////////////////////////////////////
int Train::TurnTailLightOn(int TurnOn)
{


	if (TurnOn != TailLight)
		return (ToggleTailLight());
	else 
		return 1;


}


///////////////////////////////////////////////////////////////////
//Toggles the Current State of the HeadLight and send the command to the
//systemOne board
//////////////////////////////////////////////////////////////////////
int Train::ToggleHeadLight()
{

   int instruction;
   char command[15];
   int success;
   char onoff[5];
  

   if(HeadLight == ON)
   { //Headlight is on
      instruction = 128;
      HeadLight = OFF; //Turn off
	  strcpy(onoff, "off");
   }
   else 
   {
      instruction = 144;
      HeadLight = ON; //Turn on
	  strcpy(onoff, "on");
   }

   success = makecommand(SENTCOMMAND, instruction ,Id, command);


   strcpy(DCCLastCommand, command);

 //  fout << "Train " << Id << " turned headlight " << onoff << endl;
   //command needs to be sent to the port to be executed; 
   return 1;

}


int Train::ToggleTailLight()
///////////////////////////////////////////////////////////////////
//Toggles the Current State of the TailLight and send the command to the
//systemOne board
//////////////////////////////////////////////////////////////////////
{


   int instruction;
   char command[15];
   int success;
   char onoff[5];
  

   if(HeadLight == ON){ //Taillight is on
      instruction = 130;
      HeadLight = OFF; //Turn off
	  strcpy(onoff, "off");
   }
   else {
      instruction = 129;
      HeadLight = ON; //Turn on
	  strcpy(onoff, "on");
   }

   success = makecommand(SENTCOMMAND, instruction ,Id, command);

   strcpy(DCCLastCommand, command);
   
 //  fout << "Train " << Id << " Turned Taillight " << onoff << endl;
   //command needs to be sent to the port to be executed; 
   return 1;

}

///////////////////////////////////////////////////////////////////////////////
//Changes the Direction of a train.  Remembers the oldSpeed Stops the Train toggles the
//DirectionAroundTrack. Toggles the Train's direction
//Swaps the PhotocellHas and PhotoCellNext and starts the train again with 
//remembered old speed
//////////////////////////////////////////////////////////////////////////////////
int Train::ChangeDirection(){

   int success;
   char command[15];
   int instruction;
   int oldspeed;
   char dir[15];
   char diratrack[20];

   oldspeed = Speed;


   Stop();
      
   if(Direction == FORWARD)
   {
	 Direction = REVERSE; //Change from forward to reverse
	 strcpy(dir, "reverse");
   }
   else
   {
	 Direction = FORWARD; //Change from reverse to forward
	 strcpy(dir, "Forward");
   }

   if (DirectionAroundTrack == COUNTERCLOCKWISE) //Change from counter-
   {
       DirectionAroundTrack = CLOCKWISE;
	   strcpy(diratrack, "Clockwise");
   }
   else //Change from clockwise to counterclockwise
   {
       DirectionAroundTrack = COUNTERCLOCKWISE;
	   strcpy(diratrack, "Counterclockwise");
   }

   swap(PhotocellHas, PhotocellNext);

   instruction = checkdir(Speed ,Direction);
   success = makecommand(QUEUECOMMAND, instruction ,Id, command);
      //command needs to be sent to the port to be executed; 

   strcpy(DCCLastCommand, command);

   SetSpeed(oldspeed);


  // fout << "Train " << Id << " Changed Direction to " << dir << endl;
 //  fout << "Train " << Id << " Changed Direction Around Track to " << diratrack << endl;
   return 1; //Direction not changed 


}

/////////////////////////////////////////////////////////////////////
//Reads the next photocell for a Train running on batch mode
//note that all files end with 9999 which represents a loop which begins
//at the first photocell again.  This closes, opens and begins reading 
//from the file again
////////////////////////////////////////////////////////////////////////
int Train::ReadPhotocell()
{

 return 1;

}
///////////////////////////////////////////////////////////////////////
//Returns the PhotoCell the Train Has(ie the photocell it already covered
//bounding the track that it is on
/////////////////////////////////////////////////////////////////////////
int Train::GetPhotocellHas(){

	return PhotocellHas;


}

///////////////////////////////////////////////////////////////////////
//Returns the PhotoCell the Train will have next(ie the photocell it did not covered
//bounding the track that it is on
/////////////////////////////////////////////////////////////////////////

int Train::GetPhotocellNext()
{
	return PhotocellNext;
}


///////////////////////////////////////////////////////////////////////
//Returns the Turnout the Train Has(ie the Turnout it has on the track that it is on
/////////////////////////////////////////////////////////////////////////
int Train::GetTurnoutHas()
{
	return TurnoutHas;
}





int Train::GetBlockingPhotocell()
{


	return BlockingPhotocell;

}




/////////////////////////////////////////////////////////////////////////
//Returns the last photocell covered
/////////////////////////////////////////////////////////////////////////
int Train::GetLastPhotocellCovered()
{

	return LastPhotocellCovered;

}


//////////////////////////////////////////////////////////////////////////
//Returns the Train's DirectionAroundTrack ie (Clockwise or Counterclockwise
////////////////////////////////////////////////////////////////////////////
int Train::GetDirectionAroundTrack()
{

	return DirectionAroundTrack;

}



////////////////////////////////////////////////////////////////////////////
//Returns the Trains Direction ie Forward or Reverse
////////////////////////////////////////////////////////////////////////////
int Train::GetDirection()
{

	return Direction;

}



//////////////////////////////////////////////////////////////////////
//Returns the id of a train
////////////////////////////////////////////////////////////////////
int Train::GetId()
{

	return Id; //return id

}


char * Train::GetName()
{
	return Name;

}


//////////////////////////////////////////////////////////////////////
//returns the speed of a train
////////////////////////////////////////////////////////////////////	
int Train::GetSpeed()
{

	return Speed;//return speed
}


int Train::GetHeadLight()
{
	return HeadLight;

}

int Train::GetTailLight()
{

	return TailLight;

}

////////////////////////////////////////////////////////////////////////////
//Returns the Mode ie INTERACTIVE OR BATCH of the train
///////////////////////////////////////////////////////////////////////////
int Train::GetMode(){

	return Mode;

}


void StringArrayCopy(string to[], string from[], int size)
{
	for (int i=0; i<size; i++)

		to[i] = from[i];

}


int makecommand(int type, int instruction, int id, char command[]){
/****************************************************************
* Input: type        - 's' if the command is to be sent headlight 
*						and taillight function 1 group
*			         - 'q' if the command is to be queued speed and
*						direction
*
*	     instruction - the instruction to be executed
*        id			 - the train id number
*
* Output: command - the constructed command to be executed
*         return stmt. - 1 if the command was made successfully
*					   - 0 if the command wasn't made sucessfully
*
* Purpose: To write a command that will be executed through the 
*		   open port.		   
****************************************************************/

   int checksum;  // third part of command computed by XOR of 
				  // train id and instruction
   char idchar[6];
   char instructchar[3];
   char checksumchar[8];
  

   // Note itoa changes int id into an array or chars and
   // converts them into base 16 or hexadecimal

   itoa(id, idchar, 16);   
   strcat(idchar, " ");

   itoa(instruction, instructchar, 16);
   strcat(instructchar, " ");
   // conc with a space inbetween

   checksum = id ^ instruction;

   // ^ is the XOR command in C++

   itoa(checksum, checksumchar,16);

   if (type == SENTCOMMAND)
      strcpy (command, "S C05 ");
   // the minimum number of times something must be sent is S C03

   else if (type == QUEUECOMMAND)
      strcpy (command, "Q ");

   else
      return 0;

   strcat(command, idchar);
   strcat(command, instructchar);
   strcat(command, checksumchar);
   strcat(command, "\n");
   // put them all together

   //cout << command;
 
   return 1;
}



int checkdir(int speed, int direction){
/***************************************************
* Input: speed	   - the speed of the train
*		 direction - the direction of the train
*
* Output: instruction - integer instruction
*
* Purpose: This function creates the correct integer 
*		   instruction given the speed and direction
*		   of the train.
***************************************************/

   int instruction;

   if (direction == FORWARD)
      instruction = speed + 96;  
      // note 96D = 01100000B which corresponds to the 010
      // instruction bit for forward with speed 0 see instruction
      // documentation
   else
      instruction = speed + 64;
     // note that 64D = 01000000B which corresponds to the 010
     // instruction bit for reverse with speed.

   return instruction;

}

/////////////////////////////////////////////////////////////////////////////
//Swaps two integers
////////////////////////////////////////////////////////////////////////////
void swap(int & x, int & y){

	int temp = x;
	x = y;
	y = temp;

}



/*



void main(){


	Train X(1, 16);

	X.Print();

	getch();

	X.IncreaseSpeed(2);

	X.Print();

	getch();

	X.DecreaseSpeed();

	X.Print();

	getch();

	X.ChangeDirection();

	X.Print();
}




*/

