#include "train.h"

#include <fstream>
using namespace std;


int checkdir(int speed, int direction);
int makecommand(int type, int instruction, int id, char command[]);
void swap(int & x, int & y);


Train::Train(int Id1, int Mode1, int MaxSpeed1 , int PhotocellHas1,
		 int PhotocellNext1, int TurnoutHas1, int OldPhotocellHas1, int OldTurnoutHas1,
		 int BlockingPhotocell1, int BlockingTurnout1, int DirectionAroundTrack1, 
		 int Direction1, int HeadLight1, int TailLight1){

	Id = Id1;


	MaxSpeed = MaxSpeed1;
	Mode = Mode1;


	PhotocellHas = PhotocellHas1;
	PhotocellNext = PhotocellNext1;
	TurnoutHas = TurnoutHas1;

	BlockingPhotocell = BlockingPhotocell1;
	BlockingTurnout = BlockingTurnout1;

	DirectionAroundTrack = DirectionAroundTrack1;
	Direction = Direction1;

	HeadLight = HeadLight1;
	TailLight = TailLight1;

	strcpy(DCCLastCommand, "");
	Speed = 0;


}




Train::~Train(){


}



Train::Train(const Train & T){

	 Id = T.Id;

	 Speed = T.Speed;
	 MaxSpeed = T.MaxSpeed;

	 PhotocellHas = T.PhotocellHas;
	 PhotocellNext = T.PhotocellNext;
	 TurnoutHas = T.TurnoutHas;


	 OldPhotocellHas = T.OldPhotocellHas;

	 OldTurnoutHas = T.OldTurnoutHas;


	 DirectionAroundTrack = T.DirectionAroundTrack;
	 Direction = T.Direction;

	 HeadLight = T.HeadLight;
	 TailLight = T.TailLight;

	 Mode = T.Mode;

	 BlockingPhotocell = T.BlockingPhotocell;
	 BlockingTurnout = T.BlockingTurnout;

 	 strcpy(DCCLastCommand, T.DCCLastCommand);

}




int Train::GetId(){

	return Id;

}





	
int Train::GetSpeed(){



	return Speed;


}



int Train::SetSpeed(int newspeed){


	if (newspeed > Speed){
		return (IncreaseSpeed(newspeed - Speed));
	}
	else if(newspeed < Speed)
		return (DecreaseSpeed(Speed - newspeed));
	else
		return 1;
	
}




int Train::IncreaseSpeed(){

	return(IncreaseSpeed(1));

}



int Train::IncreaseSpeed(int increment = 1){


   int success;
   int instruction;
   char command[15];
 
  
   if (Speed + increment <= MaxSpeed){//make sure not to exceed maxspeed
   
	  for (int i = 0; i<increment; i++){ 
		  // increases speed on an increment of 1

	     instruction = checkdir(Speed + 1 , Direction);
         success = makecommand(QUEUECOMMAND, instruction ,Id, command);
         //command needs to be sent to the port to be executed;
         Speed++; //increase speed in train struct
	  }//end for

	  strcpy(DCCLastCommand, command); //Update last speed command
	 
      return 1; //increased correctly
   }//end if
   else
	  return 0; //cannot increase past maxspeed
}




int Train::DecreaseSpeed(){


	return(DecreaseSpeed(1));

}




int Train::DecreaseSpeed(int increment){


   int success;
   int instruction;
   char command[15];
  
  
   if (Speed - increment >= 0){ // make sure not to go past speed zero
   
	  for (int i = 0; i<increment; i++){  // decreases speed on an increment of 1
	     instruction = checkdir(Speed - 1 , Direction);
         success = makecommand(QUEUECOMMAND, instruction ,Id, command);
         //command needs to be sent to the port to be executed;        
         Speed--; //decrease speed in train struct
	  }//end for

	  strcpy(DCCLastCommand, command); //increase speed in train struct
	 
      return 1; //decreased correctly
   }//end if
   else
      return 0; //cannot decrease past speed zero
}



int Train::Stop(){

	return (SetSpeed(0));


}



int Train::GetPhotocellHas(){

	return PhotocellHas;


}


int Train::FindNextPhotoCell(){


	return 1;



}




int Train::GetPhotocellNext(){


	return PhotocellNext;



}



int Train::GetTurnoutHas(){


	return TurnoutHas;


}





int Train::GetBlockingPhotocell(){


	return BlockingPhotocell;

}





int Train::GetBlockingTurnout(){

	return BlockingTurnout;

}



int Train::GetLastPhotocellCovered(){


	return LastPhotocellCovered;

}



int Train::GetDirectionAroundTrack(){

	return DirectionAroundTrack;




}




int Train::GetDirection(){

	return Direction;

}





int Train::TurnHeadLightOn(int TurnOn){


	if (TurnOn != HeadLight)
		return (ToggleHeadLight());
	else
		return 1;
   

}


int Train::TurnTailLightOn(int TurnOn){


	if (TurnOn != TailLight)
		return (ToggleTailLight());
	else 
		return 1;


}



int Train::ToggleHeadLight(){


   int instruction;
   char command[15];
   int success;
  

   if(HeadLight == ON){ //Headlight is on
      instruction = 128;
      HeadLight = OFF; //Turn off
   }
   else {
      instruction = 144;
      HeadLight = ON; //Turn on
   }

   success = makecommand(SENTCOMMAND, instruction ,Id, command);


   strcpy(DCCLastCommand, command);

   
   //command needs to be sent to the port to be executed; 
   return 1;

}


int Train::ToggleTailLight(){


   int instruction;
   char command[15];
   int success;
  

   if(HeadLight == ON){ //Taillight is on
      instruction = 130;
      HeadLight = OFF; //Turn off
   }
   else {
      instruction = 129;
      HeadLight = ON; //Turn on
   }

   success = makecommand(SENTCOMMAND, instruction ,Id, command);

   strcpy(DCCLastCommand, command);
   
   //command needs to be sent to the port to be executed; 
   return 1;

}


int Train::ChangeDirection(){

   int success;
   char command[15];
   int instruction;
   int oldspeed;

   oldspeed = Speed;


   Stop();
      
   if(Direction == FORWARD)
	 Direction = REVERSE; //Change from forward to reverse
   else
	 Direction = FORWARD; //Change from reverse to forward

   if (DirectionAroundTrack == COUNTERCLOCKWISE) //Change from counter-
       DirectionAroundTrack = CLOCKWISE;
   else //Change from clockwise to counterclockwise
       DirectionAroundTrack = COUNTERCLOCKWISE;

   swap(PhotocellHas, PhotocellNext);

   instruction = checkdir(Speed ,Direction);
   success = makecommand(QUEUECOMMAND, instruction ,Id, command);
      //command needs to be sent to the port to be executed; 

   strcpy(DCCLastCommand, command);

   SetSpeed(oldspeed);


   return 0; //Direction not changed 


}


void Train::Print(){
/*
    cout << "ID: " <<  Id << endl;
	cout << "Speed: " << Speed << endl;

	cout << "MaxSpeed: " << MaxSpeed << endl;

	cout << "PhotocellHas: " << PhotocellHas << endl; 
	cout << "PhotocellNext: " << PhotocellNext  << endl;
	cout << "TurnoutHas: " <<  TurnoutHas  << endl;

	cout << "BlockingPhotocell: " << BlockingPhotocell << endl;
	cout << "BlockingTurnout: " << BlockingTurnout  << endl;

	cout << "DirectionAroundTrack: " << DirectionAroundTrack << endl;
	cout << "Direction: " << Direction << endl;

	cout << "HeadLight: " << HeadLight << endl;
	cout << "TailLight: " << TailLight  << endl;

*/	
	

}


int Train::GetMode(){

	return Mode;

}


int makecommand(int type, int instruction, int id, char command[]){
/****************************************************************
* Input: type        - 's' if the command is to be sent headlight 
*						and taillight function 1 group
*			         - 'q' if the command is to be queued speed and
*						direction
*
*	     instruction - the instruction to be executed
*        id			 - the train id number
*
* Output: command - the constructed command to be executed
*         return stmt. - 1 if the command was made successfully
*					   - 0 if the command wasn't made sucessfully
*
* Purpose: To write a command that will be executed through the 
*		   open port.		   
****************************************************************/

   int checksum;  // third part of command computed by XOR of 
				  // train id and instruction
   char idchar[6];
   char instructchar[3];
   char checksumchar[8];
  

   // Note itoa changes int id into an array or chars and
   // converts them into base 16 or hexadecimal

   itoa(id, idchar, 16);   
   strcat(idchar, " ");

   itoa(instruction, instructchar, 16);
   strcat(instructchar, " ");
   // conc with a space inbetween

   checksum = id ^ instruction;

   // ^ is the XOR command in C++

   itoa(checksum, checksumchar,16);

   if (type == SENTCOMMAND)
      strcpy (command, "S C05 ");
   // the minimum number of times something must be sent is S C03

   else if (type == QUEUECOMMAND)
      strcpy (command, "Q ");

   else
      return 0;

   strcat(command, idchar);
   strcat(command, instructchar);
   strcat(command, checksumchar);
   strcat(command, "\n");
   // put them all together

   //cout << command;
 
   return 1;
}



int checkdir(int speed, int direction){
/***************************************************
* Input: speed	   - the speed of the train
*		 direction - the direction of the train
*
* Output: instruction - integer instruction
*
* Purpose: This function creates the correct integer 
*		   instruction given the speed and direction
*		   of the train.
***************************************************/

   int instruction;

   if (direction == FORWARD)
      instruction = speed + 96;  
      // note 96D = 01100000B which corresponds to the 010
      // instruction bit for forward with speed 0 see instruction
      // documentation
   else
      instruction = speed + 64;
     // note that 64D = 01000000B which corresponds to the 010
     // instruction bit for reverse with speed.

   return instruction;

}


void swap(int & x, int & y){

	int temp = x;
	x = y;
	y = temp;

}



/*



void main(){


	Train X(1, 16);

	X.Print();

	getch();

	X.IncreaseSpeed(2);

	X.Print();

	getch();

	X.DecreaseSpeed();

	X.Print();

	getch();

	X.ChangeDirection();

	X.Print();
}




*/

