// ccr.cpp 


//#include <windows.h>
//#include <iostream.h>
//#include <fstream.h>
//#include <stdio.h>
#include "ccr.h"
#include  "Pplib95.h"




//***************************************************************//
//                                                               //
//                       BASE FUNCTIONS                          //
//                                                               //
//***************************************************************//

//***************************************************************
//Opens the Peek and Poke Driver.  Returns TRUE if successful.  
//This must be called before any calls are made to other modules.
//***************************************************************

short _stdcall Open_Ports ()  {

   short x;

   x = ics_pp_open ();
   return x;
}


//***************************************************************
//Closes the driver. Should be called before the application
//exits.
//***************************************************************

void _stdcall Close_Ports ()  {

   ics_pp_close ();
}


//***************************************************************
//Initailizes the board (1-9) to input and output sections based
//on the input parameters 0 and 1 for A, B, CL, and CH.
//Returns: 1 when successful and 0 when error
//***************************************************************

short _stdcall Initialize_Board (short board_num, short a_val, short b_val,
		      short cl_val, short ch_val)  {

   short x;			//working value
   short ok;			//flag for success

   //check for valid I/O values
   if (((a_val != 1) && (a_val != 0)) ||
       ((b_val != 1) && (b_val != 0)) ||
      ((ch_val != 1) && (ch_val != 0)) ||
      ((cl_val != 1) && (cl_val != 0)))

      return (0);

   //store new values
   x = 1;                      	//final digit format = "1 0 0 A, CH 0 B CL"
   x = x << 3;
   x = x + a_val;     		    //set a_val  - 0 0 0 0, 1 0 0 A
   x = x << 1;
   x = x + ch_val;              //set ch_val - 0 0 0 1, 0 0 A CH
   x = x << 2;
   x = x + b_val;              	//set b_val  - 0 1 0 0, A CH 0 B
   x = x << 1;
   x = x + cl_val;             	//set cl_val - 1 0 0 A, CH 0 B CL

   //put value out to port
   ok = Set_Port ((TO_output-1) + (board_num * 4), x);

   return (ok);
}

//***************************************************************
//Sets the channel of a certain port based on the input parameters
//to the 'ON' position.  In the case of the I/O Board, a 1 is off
//while a 0 is on.
//Returns 1 if successful and 0 if errors.
//***************************************************************

short _stdcall Set_Chan_On(short port_num, short chan_num)  {

   short portval;			//value of the port
   short new_portval;		//new value of the port to be set
   short x = 1;				//modifier for byte value

   //verify that channel is valid
   if ((chan_num < 0) || (chan_num > 7))
      return (0);			//not a valid channel

   //modify portval
   portval = Get_Port (port_num);    //get the value of the port
   x = x << chan_num;			//shift bits left "chan_num" places
   x = ~ x;           	        //flips bits "11110111"
   new_portval = portval & x;   //turns chan_num bit to "0/on"

   //put new_portval out to port and verify it
   x = Set_Port (port_num, new_portval);

   return (x);
}

//***************************************************************
//Sets the channel of a certain port based on the input parameters
//to the 'OFF' position.  In the case of the I/O Board, a 1 is off
//while a 0 is on.
//Returns 1 if successful and 0 if errors.
//***************************************************************

short _stdcall Set_Chan_Off(short port_num, short chan_num)  {

   short portval;			//value of the port
   short new_portval;		//new value of the port to be set
   short x = 1;				//modifier for byte value

   //verify that channel is valid
   if ((chan_num < 0) || (chan_num > 7))
      return (0);			//not a valid channel

   //modify portval
   portval = Get_Port (port_num);     //get port value
   x = x << chan_num;			//shift bits left "chan_num" places
   new_portval = portval | x;   //turns chan_num bit to "1/off"

   //put new_portval out to port and verify it
   x = Set_Port (port_num, new_portval);

   return (x);
}

//***************************************************************
//Gets the decimal value of a channel of a certain port based
//on the input parameters.  Manipulates this port to determine
//if it is on or off.  This module uses C++ logic in that a 1
//is considered on and a 0 is off.
//Returns 1 if 'ON' or 0 if channel is 'OFF'.
//***************************************************************

short _stdcall Get_Channel(short port_num, short chan_num)  {

   short portval;			//value of the port
   short x = 1;				//modifier for byte value

   portval = Get_Port (port_num);		//get port value
   portval = portval >> chan_num;		//isolate "chan" bit
   x = portval & x;			//check chan_num bit (0 or 1)
   x = x ^ 1;      			//flip bit to be C++ logic

   return (x);            	// 1 = ON, 0 = OFF
}

//***************************************************************
//Gets the decimal value of a port located on our board.  The
//port to check is sent in through the parameter port_num.
//Returns the decimal value of the port.
//***************************************************************

short _stdcall Get_Port(short port_num)  {

   short portval;				//value of the port

   portval = _inp (port_num); 	//get port value
   return (portval);   			//returns the value of the port
}

//***************************************************************
//Sets the decimal value of a port located on our board.  The
//port to set is sent in through the parameter port_num and the
//value to set is passed in through portval.
//Returns 1 if succesful and 0 if errors.
//***************************************************************

short _stdcall Set_Port(short port_num, short portval)  {

   short value;					//value of the port

   //store new_portval to port
   value = _outp (port_num, portval);

   //verify that it was stored correctly
   if (value != portval)
      return (0);				//not correctly stored

   return (1);            		//success

}


//***************************************************************
//Turns a Turnout to the 'ON' position. The turnout to switch
//is sent in through the parameter TO_num and corresponds to
//the number on the I/O Board.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************

short _stdcall TO_On (short TO_num)  {

   short port_num;		//port to be set
   short chan_num;		//channel to be turned on
   short ok;			//flag for successful operation

   //calculate the port and channel
   port_num = TO_output + (TO_num / 8);
   chan_num = TO_num % 8;

   //set the appropriate channel to on
   ok = Set_Chan_On (port_num, chan_num);

   return (ok);
}


//***************************************************************
//Turns a Turnout to the 'OFF' position. The turnout to switch
//is sent in through the parameter TO_num and corresponds to
//the number on the I/O Board.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************

short _stdcall TO_Off (short TO_num)  {

   short port_num;		//port to be set
   short chan_num;		//channel to be turned off
   short ok;			//flag for successful operation

   //calculate the port and channel
   port_num = TO_output + (TO_num / 8);
   chan_num = TO_num % 8;

   //set the appropriate channel to off
   ok = Set_Chan_Off (port_num, chan_num);

   return (ok);
}

//***************************************************************
//Toggles a Turnout to the opposite position. The turnout to switch
//is sent in through the parameter TO_num and corresponds to
//the number on the I/O Board.
//Returns a 1 if successful and a 0 if errors.
//***************************************************************

short _stdcall TO_Toggle (short TO_num)  {

   short port_num;		//port to be switched
   short chan_num;		//channel to be switched
   short x;       		//current status of channel
   short ok;			//flag for successful operation

   //calculate the port and channel
   port_num = TO_output + (TO_num / 8);
   chan_num = TO_num % 8;

   //set the appropriate channel to switch
   x = Get_Channel (port_num, chan_num);
   if (x == 1)
      ok = Set_Chan_Off (port_num, chan_num);
   else if (x == 0)
      ok = Set_Chan_On (port_num, chan_num);

   return (ok);
}



//***************************************************************
//Gets the position of a turnout.
//Returns:   position A, value = -1
//           position B, value =  1
//               middle, value =  0
//***************************************************************

short _stdcall Get_TO_Status (short TO_num)  {

   short port_num;		//port to be looked at
   short chan_num;		//channel to be looked at
   short a_val;			//value of channel at position A
   short b_val;         //value of channel at position B
   short position;

   //get the channel value for position straight
   port_num = TO_input_straight + (TO_num / 8);
   chan_num = TO_num % 8;
   a_val = Get_Channel (port_num, chan_num);

   //get the channel value for position curved
   port_num = TO_input_curved + (TO_num / 8);
   chan_num = TO_num % 8;
   b_val = Get_Channel (port_num, chan_num);

   //return position of the turnout
   if (a_val > b_val)    position = -1;	//position staight
   if (a_val < b_val)    position =  1;	//position curved
   if (a_val == b_val)   position =  0;	//middle

   return (position);
}
   
   
//***************************************************************
//Gets the status of a optical sensor.
//Returns:    ON, value = 1
//           OFF, value = 0
//***************************************************************

short _stdcall Get_Sensor_Status (short sensor_num)  {

	short port_num;		//port to be looked at
    short chan_num;		//channel to be looked at
	short status;		//status of the sensor returned by Get_Channel

	chan_num = sensor_num % 8;
	
	//sensor_num/24 = number of control ports to skip
	//sensor_num/8 =  number of 8 bit ports needed (without control ports)
	port_num = (Sensor_input_1 + (sensor_num/24) + (sensor_num/8));

	status = Get_Channel (port_num, chan_num);

	return (status);	//ON  = 1
						//OFF = 0
}

