/* Kratz Multi-Process System Client APIs       */
/* These Api Functions will provide easy access */
/* To the functionality built into the          */
/* multi-process system server. Anyone wishing  */
/* to write a client for a system based on the  */
/* Multi-process system server is strongly      */
/* to use these API's                           */ 
/* ############################################ */
/*             :|         _____            |:   */
/*    -+------,____     ,"     ",          |:   */
/*   | |------|____\_  /  _    _ \         |:   */
/* ----|      |      \'  |#|  |#| ',       |:   */
/* ----|      |    () |  |#|  |#|  |       |:   */
/* ----|      |______/   |#|  |#|  |       |:   */
/*   | |      |       ".    __    ."       |:   */
/*   "-+------":|       \  |##|  /         |:   */
/*             :|        ".____."          |:   */
/*             :|                          |:   */
/* ############################################ */
/* This file is the implementations of the API  */
/* functions                                    */
/*                                              */
/* Created: March 7, 2007 Christopher Kratz SNC */
/*                                              */
/* Modfication Log:                             */
/*                                              */
/* ############################################ */

#include <iostream>
#include <windows.h>
#include "kmp_client_api.h"
using namespace std;    // use the standard namespace



HANDLE ConnectToServer(char * PipeName)
{
	/* Connects to the system server on the specified pipename */
	/*#########################################################*/
	/*                                                         */
	/*                                                         */
	/*                                                         */
	/*      ===========                                        */
	/*     #*#   -  - -#=======================================*/
	/*    #%**# ------- #                 \__/\                */
	/*   #%%***#  - - -  #   -- -- -- -- -/ --/^\-  --- -- --- */
	/*   #%%***#         #  ---------------------------------- */
	/*  #%%%%***#         #  #### ##### #### #### ####  ##     */
	/*  #%%%%***#         #  #  #   #   #  # #    #     ##     */
	/*   #%%***#         #   ####   #   #### #### ####  ##     */
	/*   #%%***#         #   #      #   #    #       #     -- -*/
	/*    #%**# -- -- --#    #    ##### #    #### #### -##-  - */
	/*     #*#   -- - -#=======================================*/
	/*      ===========             ____    _____              */
	/*     /%\                  ___/%%%%\--/%%%%%\             */
	/*     \%/                 /%%%%%/\%%%%%%%%%%/             */
	/*#########################################################*/
	/* Wait For the server to open the named pipe */


	int ret =0;
	while(ret==0)
	{
		ret = WaitNamedPipe(PipeName, NMPWAIT_WAIT_FOREVER); /* Waits 100ms for available pipe instance
																returns immediately if pipe is not yet created */
		
		Sleep(100);							/*Sleep 100 ms before trying again */
	}

	/* An outstanding ConnectNamedPipe call has been made on the server side,
		the pipe can now be connected using the CreateFile functino		*/

	HANDLE pipe = CreateFile(	PipeName,
								GENERIC_WRITE | GENERIC_READ,
								0,
								NULL,
								OPEN_EXISTING,
								FILE_ATTRIBUTE_NORMAL,
								NULL);


	/* Returns INVALID_HANDLE_VALUE if CreateFile call fails, not much to do here */

	if (pipe == INVALID_HANDLE_VALUE )
	{
		/*Pipe Connection Failed*/
		return pipe;
	}

	/* Return the handle to the open pipe */
	return pipe;
}

/*#####################################################################################*/
/*#####################################################################################*/
/*#####################################################################################*/



int GetMessageFromPeer(char * From, unsigned long& ServiceCode, char * MessageData, unsigned long& MessageLength, HANDLE pipe)
{
	/*######################################################*/
	/* GETS A MESSAGE FROM CLIENTS QUEUE and returns the	*/
	/* number of messages left in queue						*/
	/*	or -1 if there was no message to recieve			*/
	/*######################################################*/
	/*######################################################*/
	/*                         __                           */
	/*                        | |                           */
	/*                     .-----.--------------------.     */
	/*                    / ##### \.                   \.   */
	/*               ___________ ## \                    \  */
	/*          .---|%%%      %%|###|                    |  */
	/*  -------/   \|           |###|   .------------    |  */
	/*              '--  %%%%   |###|  |   ________. |   |  */
	/*            \ ___) %%%%   |###|  | *|        | |   |  */
	/*  	        |___________|###|  "--"        \_|   |  */
	/*  ---------,_____/ ###########|                    |  */
	/*            ______|___________|                    |  */
	/*	         |------////////////'""""""""""""""""""""'  */
	/*	          "/#/"""""""""""""       |=.|.:|           */
	/*	          (#/                     |=.|.:|           */
	/*                                    |=.|.:|           */
	/*                                    |=.|.:|           */
	/*                                                      */
	/*######################################################*/


	/* Buffer To Send Request for message */
	char newmessage[2];

	/* Buffer to recieve message */
	char buffer[CLIENT_SIZE_MESSAGE];

	newmessage[0] = 0x01;

	/* Bytes Read */
	unsigned long bytes;
	

	/* Write Request to the pipe */
	if(WriteFile(pipe, newmessage, 1, &bytes, NULL) == 0)
	{
		/*Write Failed*/
		cout << "Write failed.\n";
		if(GetLastError() == ERROR_BROKEN_PIPE)
		{
			/*Broken  pipe condition */
			cout << "BROKEN PIPE!!!!!\n";
		}
	}

	/* Read Response */
	if(ReadFile(pipe, buffer, 255, &bytes, NULL)==0)
	{
		/* Read failed */
		cout << "Read Failed.\n";
		return -2;
	}
	if(bytes == 1)
	{
		/* No messages in queue */
		if(buffer[0]== 0x01) return -1;
	}

	if(bytes < 6)
	{ 
		/* Improperly formatted message */
		return -3;
	}

	// response data format
	// [RETURN CODE x2 BYTES] [REMAINING MESSAGES x2 BYTES] [SERVICE CODE x2 BYTES] [FROM] 0x00 [MESSAGE DATA]

	int ret_code;
	int messages_left;
	
	/* Return code 16 bits */
	ret_code = (int)buffer[0] * 256;
	ret_code += (int)buffer[1];


	/* Messages left 16 bits */
	messages_left = (int)buffer[2] * 256;
	messages_left = (int)buffer[3];

	/* Service code 16 bits */
	ServiceCode = (int)buffer[4] * 256;
	ServiceCode = ServiceCode + (int)buffer[5];

	/* From */
	strncpy(From, &buffer[6], bytes);

	/*Message Length*/
	MessageLength = bytes - 6 - strlen(From)-1;
	
	int offset = 6+1+strlen(From);

	unsigned int i;
	for(i=0; i<bytes; i++)
	{
		MessageData[i] = buffer[i+offset];
	}

	return messages_left;

}

/*#####################################################################################*/
/*#####################################################################################*/
/*#####################################################################################*/


bool SendMessageToPeer(char * MessageData, unsigned long Length, unsigned long ServiceCode, char * Destination, HANDLE pipe)
{
	/*######################################################*/
	/* SENDS A MESSAGE TO ANOTHER CLIENTS QUEUE         	*/
	/* returns false if it failed                           */	
	/*	                                                    */
	/*######################################################*/
	/*######################################################*/
	/*                         __        .____              */
	/*                        | |        |    '             */
	/*                     .-----.-------|    |-------.     */
	/*       _           ./  | |  \.     | +--"        \.   */
	/*      | |         /           \    | |             \  */
	/*      | | __      |           |:   | |             |  */
	/* -----+ ":__|     |  ~~~~~~~  |:   | |             |  */
	/*         :__|     |           |:  .| |.            |  */
	/*         :__|     |           |:  | * |            |  */
	/* --------:__|     | US MAIL   |:  |___|            |  */
	/*                  |           |:                   |  */
	/*                  |___________|:                   |  */
	/*	                 ~~~~~~~~~~~'""""""""""""""""""""'  */
	/*	                                  |=.|.:|           */
	/*	                                  |=.|.:|           */
	/*                                    |=.|.:|           */
	/*                                    |=.|.:|           */
	/*                                                      */
	/*######################################################*/

	/* Send Message   */
	/* data format    */
	/*  Ox00 [DESTINATION NAME] Ox00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES] */

	/* buffer to hold data to be sent on pipe */
	char newmessage[CLIENT_SIZE_MESSAGE + CLIENT_SIZE_CLIENTNAME + 6];


	/* Send message command is 0x00 */
	newmessage[0] = 0x00;
	/* copy destination name into buffer */
	strncpy(&newmessage[1], Destination, CLIENT_SIZE_CLIENTNAME);
	
	/* Calculate offset in data */
	int offset = strlen(Destination) + 1;

	/* 0x00 termniates the destination field */
	newmessage[offset] = 0;

	/* Service code 1 */
	newmessage[offset+1] = ServiceCode / 256;
	newmessage[offset+2] = ServiceCode % 256;

	/* Service code adds 2 more bytes plus null terminator for destination field */
	offset += 3;

	/* Copy message data into buffer */
	unsigned int i;
	for(i=0;i<Length;i++)
	{
		newmessage[i+offset] = MessageData[i];
	}
	/* Total number of bytes placed in buffer */
	offset += i;

	/* FINISHED filling buffer */
	
	unsigned long bytes;

	/* Write buffer to the pipe */
	if(WriteFile(pipe, newmessage, offset, &bytes, NULL) == 0)
	{
		/* Failure !! */
		cout << "Write failed. \n";
		return false;
	}

	/* Read Acknowledge from pipe */
	char buffer[CLIENT_SIZE_BUFFER];
	if(ReadFile(pipe, buffer, 255, &bytes, NULL)==0)
	{
		/* Failure !! */
		cout << "Read Failed.\n";
		return false;
	}

	/* Server should return zero for success */
	/* Currently this is the only defined response */
	if(bytes == 1)
	{
		if(buffer[0] == 0x00)
		{
			return true;
		}
		else if(buffer[1] == 0x01)
		{
			return false;
		}
		else
		{
			return false;
		}
	}
	else
	{
		return false;
	}


}

bool SendMulticastMessageToPeer(char * MessageData,					/* Data that you are sending */ 
					   unsigned long Length,						/* Length of the data that you are sending */
					   unsigned long ServiceCode,					/* Service Code for the data you are sending */
					   char Destination[][CLIENT_SIZE_CLIENTNAME],	/* Destinations for your message */
					   unsigned long NumberDestinations,			/* Number of destinations */
					   HANDLE pipe)									/* Handle to open pipe */
{
	/*######################################################*/
	/* SENDS A MESSAGE TO ANOTHER CLIENTS QUEUE         	*/
	/* returns false if it failed                           */	
	/*	                                                    */
	/*######################################################*/
	/*######################################################*/
	/*                         __        .____              */
	/*                        | |        |    '             */
	/*                     .-----.-------|    |-------.     */
	/*       _           ./  | |  \.     | +--"        \.   */
	/*      | |         /           \    | |             \  */
	/*      | | __      |           |:   | |             |  */
	/* -----+ ":__|     |  ~~~~~~~  |:   | |             |  */
	/*         :__|     |           |:  .| |.            |  */
	/*         :__|     |           |:  | * |            |  */
	/* --------:__|     | US MAIL   |:  |___|            |  */
	/*                  |           |:                   |  */
	/*                  |___________|:                   |  */
	/*	                 ~~~~~~~~~~~'""""""""""""""""""""'  */
	/*	                                  |=.|.:|           */
	/*	                                  |=.|.:|           */
	/*                                    |=.|.:|           */
	/*                                    |=.|.:|           */
	/*                                                      */
	/*######################################################*/

	/* Send Message   */
	/* data format    */
	//  Ox02 [NUMBER OF DESTINATIONS x2 BYTES] [DESTINATION 1] Ox00 [DESTINATION 2] 0x00 .... [DESTNATION N] 0x00 [SERVICE CODE X2 BYTES] [MESSAGE BYTES]

	/* buffer to hold data to be sent on pipe */
	char newmessage[CLIENT_SIZE_MESSAGE + CLIENT_SIZE_CLIENTNAME   + 6];


	/* Send message command is 0x00 */
	newmessage[0] = 0x00;
	/* copy destination name into buffer */
	
	int offset = 1;
	unsigned long i;
	for(i=0;i<NumberDestinations;i++)
	{
	
		strncpy(&newmessage[offset], Destination[i], CLIENT_SIZE_CLIENTNAME);
		
		/* Calculate offset in data */
		int offset = strlen(Destination[i]) + 1;

		/* 0x00 termniates the destination field */
		newmessage[offset] = 0;
		offset++;
	}

	/* Service code 1 */
	newmessage[offset+1] = ServiceCode / 256;
	newmessage[offset+2] = ServiceCode % 256;

	/* Service code adds 2 more bytes plus null terminator for destination field */
	offset += 2;

	/* Copy message data into buffer */
	
	for(i=0;i<Length;i++)
	{
		newmessage[i+offset] = MessageData[i];
	}
	/* Total number of bytes placed in buffer */
	offset += i;

	/* FINISHED filling buffer */
	
	unsigned long bytes;

	/* Write buffer to the pipe */
	if(WriteFile(pipe, newmessage, offset, &bytes, NULL) == 0)
	{
		/* Failure !! */
		cout << "Write failed. \n";
		return false;
	}

	/* Read Acknowledge from pipe */
	char buffer[CLIENT_SIZE_BUFFER];
	if(ReadFile(pipe, buffer, 255, &bytes, NULL)==0)
	{
		/* Failure !! */
		cout << "Read Failed.\n";
		return false;
	}

	/* Server should return zero for success */
	/* Currently this is the only defined response */
	if(bytes == 1)
	{
		if(buffer[0] == 0x00)
		{
			return true;
		}
		else if(buffer[1] == 0x01)
		{
			return false;
		}
		else
		{
			return false;
		}
	}
	else
	{
		return false;
	}


}

/*####################################################################################*/
/*                                                                                    */
/*    %%%%  %%%%   %%%%%%%    %%%%%%%%%  %%%%%%%%%  %%%%%%%%%                         */
/*    %%%%  %%%%   %%%%%%%%   %%%   %%%  %%%%%%%%%  %%%%%%%%%                         */
/*   %%%%%%%%     %%%%  %%%  %%%%%%%%%     %%%        %%%%%                           */
/*   %%%%%%%%     %%%%%%%%   %%%%%%%%%     %%%      %%%%                              */
/*  %%%%  %%%%   %%%% %%%   %%%   %%%     %%%     %%%%%%%%%                           */
/*  %%%%  %%%%   %%%  %%%%  %%%   %%%     %%%     %%%%%%%%%                           */
/*                                                                                    */
/*      #######   ####    ####    ########       ****     ***     ***    ******       */
/*     #########  #####   ####   ##########     ******   *****   *****   *******      */
/*    ####        ######  ####  ####    ###          *  **   ** **   **       **      */
/*    ####        ######  ####  ####               **   **   ** **   **      **       */
/*     #######    ####### ####  ####              **    **   ** **   **     **        */
/*      #######   ############  ####            ******   *****   *****     **         */
/*          ####  #### #######  ####            ******    ***     ***     **          */
/*          ####  ####  ######  ####    ###                                           */
/*    #########   ####   #####   ##########      -- ---  !  ! !    +--+               */
/*     #######    ####    ####    ########      |   ",   +--! !--+ |  |               */
/*                                               -- --"     ! !__! +--+               */
/*####################################################################################*/