﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.IO.Pipes;                  //needed for named pipes
using System.Runtime.InteropServices;   //needed for p/invoke
using Microsoft.Win32.SafeHandles;      //also needed for pipes and pipehandles/IntPtrs
using System.IO;

//***************************************BreakDown of CCR*********************************************
//This program relies on on two exe's, DccMan and test as worker threads, dccman for track reporting
//and test for pipe creation to allow for inter-process communication.  It also relies on the tos_api.dll
//for some unmanaged c++ code so that we can utilize some prior written code (Kratz).  This program talks directly
//thru the pipe to the dccman.exe which then parses and commands the powerhouse of the train.  The dccman 
//source code should be included in the source folder, and has been brought into the VS2008 arena so that it 
//can cooperate with the c# more.  For starters, all track controls are 'manipulated' here in c#, and 
//then called using the p/invoke method to be sent to the DCCman so it can send the powerhouse the correct
//commands.  The DCCman handles all of the port control.  TOS_API.c will be a good reference if you need to 
//know how the commands get from here to the DCCman.  PortSelect.cs simply determines what com port you 
//are using for the powerhouse. Joy.cs handles all of the gamepad controls and train/track manipulation.
//****************************************************************************************************


namespace CCR
{
    public partial class Receive : Form
    {   //******************************************************************************************
        //[DllImport(".dll")]           //P/INVOKE STANDARD LAYOUT
        //public static extern void ();
        //-----------------------------------------------------------------------------------------
        //P/Invoke is used for calling previously written code by use of .dll. You 'import' the function
        //prototype thru the calls below and the code is actually referenced in the .dll included
        //the .dll included was constructed in visual6.0 using the header, cpp, and a definition file
        //and should all be included in source code.
        //******************************************************************************************

        [DllImport("tos_api.dll")]  //Initializes DCC for sending/receiving using tos_api.dll
        public static extern bool TOS_InitDCC(string port, IntPtr pipe);

        [DllImport("kmp_client_api.dll", EntryPoint = "SendMessageToPeer")]//Sends messages to thread clients
        public static extern bool SendMessageToPeer(string message, UInt16 length, UInt16 code, string Destination, IntPtr pipe);
        //SendMessageToPeer(char * MessageData, unsigned long Length, unsigned long ServiceCode, char * Destination, HANDLE pipe)

        [DllImport("tos_api.dll")]//Reads the AIU's on the track for the track status
        public static extern bool TOS_ReadAIU(uint address, uint input, IntPtr pipe);
        //bool TOS_ReadAIU(unsigned long Address, unsigned long Input, HANDLE pipe);

        [DllImport("tos_api.dll")]//Modifies train engine speed and direction
        public static extern bool TOS_SetTrainSpeedDCC(uint TrainAddress, uint Speed, byte direction, IntPtr pipe);
        //TOS_SetTrainSpeedDCC(unsigned long TrainAddress, unsigned long Speed, unsigned char direction, HANDLE pipe)

        [DllImport("tos_api.dll")]//Sets turnout left or right at the address specified
        public static extern bool TOS_SetTurnout(uint Address, byte value, IntPtr pipe);
        //TOS_SetTurnout(unsigned long Address, unsigned char value, HANDLE pipe)

        [DllImport("kmp_client_api.dll")]//Gets the message waiting on the TOS_UI pipe (from dccman)
        public static extern int GetMessageFromPeer(ref byte From, ref uint ServiceCode, ref byte MessageData, ref uint MessageLength, IntPtr pipe);
        //int GetMessageFromPeer(char * From, unsigned long& ServiceCode, char * MessageData, unsigned long& MessageLength, HANDLE pipe)

        string select;   //Initializes select as public  
        string[] procs = new string[5];
        ProcessStartInfo test = new ProcessStartInfo(); //process that creates all the pipes
        ProcessStartInfo DCCMan = new ProcessStartInfo();   //Starts the DCCMan worker thread for track control
        Process exe1;   //this will become test.exe
        Process exe2;   //and this DCCMan.exe
        string[] aiuStatusMessage = new string[4];
        NamedPipeClientStream peterpiper = new NamedPipeClientStream(".", "TOSUI", PipeDirection.InOut);
        /*^^^peterpiper is the handle to our beloved pipe so that we can communicate with worker threads^^^*/


        //******************************START UP PROCESSES**********************************
        public Receive(string from) //from is the port that was selected from the previous form PortSelect.cs
        {
            //Sets up current form
            InitializeComponent();
            select = from;
            comPortID.Text = "Using: " + select;
            trainIdBox.Text = "3";              //Train Address
            turnoutListBox.Items.Add("1");      //Turnout Addresses_
            turnoutListBox.Items.Add("2");                          //
            turnoutListBox.Items.Add("150");                        //
            turnoutListBox.Items.Add("151");                        //
            turnoutListBox.Items.Add("152");                        //
            turnoutListBox.Items.Add("162");    //_________________//
            turnoutListBox.SelectedIndex = 0;
            
            if (startUp())
            {
                connectToPipe();    //Call to connectPipe function written below
                initDCC();          //Initializes the DCC manager to be talked to
            }
            else
            { MessageBox.Show("Failure to start threads"); Application.Exit(); }
        }

        private void Receive_FormClosing(object sender, FormClosingEventArgs e)
        {   //KILLS THREADS
            stopButton_Click(sender, e);    //stop train on close
            try
            { exe2.Kill(); }
            catch (Exception exc)
            { MessageBox.Show("Problem Closing Thread" + exc.ToString()); }
            try
            { exe1.Kill(); }
            catch (Exception exc)
            { MessageBox.Show("Problem Closing Thread" + exc.ToString()); }
        }

        private void Receive_FormClosed(object sender, FormClosedEventArgs e)
        {
            Application.Exit(); //Kills application when app is closed
        }
      
        public bool startUp()
        {//Starts threads
            try
            {
                test.FileName = "test.exe";
                test.WindowStyle = ProcessWindowStyle.Hidden;   //hides the cmd window
                exe1 = Process.Start(test); //starts process included in Solution Explorer

                DCCMan.FileName = "DCCMan.exe";
                DCCMan.WindowStyle = ProcessWindowStyle.Hidden;
                exe2 = Process.Start(DCCMan);
                return true;
            }
            catch
            { return false; }
        }


        private void quitButton_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void connectToPipe()
        {
            try
            {peterpiper.Connect();} //Connects to TOSUI pipe created by test.exe worker thread
            catch (Exception ex)
            {MessageBox.Show("Problem Connecting " + ex.ToString());}
        }
        //***************************End of Startup*************************************

        //**********************TRAIN MANAGEMENT****************************************
        private void setTrainSpeedButton_Click(object sender, EventArgs e)
        {
            try
            {
                UInt16 trainId = Convert.ToUInt16(trainIdBox.Text); //train address text box
                UInt16 speed = Convert.ToUInt16(speedBox.Text);     //train speed text box
                byte direction = Convert.ToByte(directionButton.Checked);   //train direction check box
                TOS_SetTrainSpeedDCC(trainId, speed, direction, peterpiper.SafePipeHandle.DangerousGetHandle());
            }
            catch(Exception ex)
            {MessageBox.Show("Failure to set train speed " + ex.ToString());}
        }


        private void stopButton_Click(object sender, EventArgs e)
        {//Stops train by setting speed to 0
            try
            {
                speedSlider.Value = 0;
                UInt16 trainId = Convert.ToUInt16(trainIdBox.Text);
                UInt16 speed = 0;
                byte direction = Convert.ToByte(directionButton.Checked);
                TOS_SetTrainSpeedDCC(trainId, speed, direction, peterpiper.SafePipeHandle.DangerousGetHandle());
            }
            catch (Exception ex)
            { MessageBox.Show("Failure to set train speed " + ex.ToString());}
        }

        private void speedSlider_MouseCaptureChanged(object sender, EventArgs e)
        {   //this is the sliderbar control of the train, direction and speed are controlled here
            //MessageBox.Show(speedSlider.Value.ToString());
            try
            {
                UInt16 trainId = Convert.ToUInt16(trainIdBox.Text);
                Int16 speed = Convert.ToInt16(speedSlider.Value);
                byte direction = 0;
                if (speed < 0)
                {
                    direction = 0;  //backward
                    speed = Convert.ToInt16(speed * -1);
                    dirLabel.Text = "BKWD:";
                    speedLabel.Text = speed.ToString();
                }
                else
                {
                    direction = 1;
                    dirLabel.Text = "FWD:";
                    speedLabel.Text = speed.ToString();
                }
                UInt16 TrainSpeed = Convert.ToUInt16(speed);
                TOS_SetTrainSpeedDCC(trainId, TrainSpeed, direction, peterpiper.SafePipeHandle.DangerousGetHandle());
            }
            catch (Exception ex)
            { MessageBox.Show("Failure to set train speed " + ex.ToString()); }
        }

        private void speedSlider_ValueChanged(object sender, EventArgs e)
        {//anytime the slider is moved, keyboard, mouse....etc
            speedSlider_MouseCaptureChanged(sender, e);
        }
        //*************************End of train mgmt*****************************************

        //*********************************TURNOUT MANAGEMENT****************************
        byte dir = 0;
      
        private void turnoutSwitchButton_Click(object sender, EventArgs e)
        {
            dir = Convert.ToByte(turnOutDirButton.Checked); //the Swittch...Checkbox for direction
            TOS_SetTurnout(Convert.ToUInt32(turnOutAddBox.Text), dir, peterpiper.SafePipeHandle.DangerousGetHandle());
        }

        private void turnoutLeftButton_Click(object sender, EventArgs e)    //turns selected turnout left
        {TOS_SetTurnout(Convert.ToUInt32(turnoutListBox.Items[turnoutListBox.SelectedIndex].ToString()), 0, peterpiper.SafePipeHandle.DangerousGetHandle()); }

        private void turnoutRightButton_Click(object sender, EventArgs e)   //turns selected turnout right
        { TOS_SetTurnout(Convert.ToUInt32(turnoutListBox.Items[turnoutListBox.SelectedIndex].ToString()), 1, peterpiper.SafePipeHandle.DangerousGetHandle());}

        private void addTurnoutButton_Click(object sender, EventArgs e)     //adds the address in the Turnout Address textbox to the list
        {turnoutListBox.Items.Add(turnOutAddBox.Text); }

        private void removeTurnoutButton_Click(object sender, EventArgs e)  //removes selected address from turnoutlist
        {turnoutListBox.Items.RemoveAt(turnoutListBox.SelectedIndex);}

        private void button1_Click(object sender, EventArgs e)
        {//this resets all the turnouts to the right
            for (int i = 0; i < turnoutListBox.Items.Count; i++)
            {
                TOS_SetTurnout(Convert.ToUInt32(turnoutListBox.Items[i].ToString()), 0, peterpiper.SafePipeHandle.DangerousGetHandle());
                System.Threading.Thread.Sleep(200);
                Application.DoEvents();
            }
        }

        //******************************end of turnout mgmt***************************************

        //*******************************DCC MANAGEMENT*********************************
        private void initDCC()
        {
            string port = select;

            if (TOS_InitDCC(port, peterpiper.SafePipeHandle.DangerousGetHandle()))
            {
                byte[] from = new byte[255];    //this will hold the confrim message sender back from the DCC
                uint service = 0;                       //this will hold the service code back from the DCC
                byte[] messagedata = new byte[16384];   //this will hold the actuall message
                uint messagelength = 0;                 //and length
                string mess = "";
                char test = ' ';

                while (test != 'D') //Catches the D of the DCCMan name tag to check port state
                {
                    try
                    {
                        GetMessageFromPeer(ref from[0], ref service, ref messagedata[0], ref messagelength, peterpiper.SafePipeHandle.DangerousGetHandle());
                        ASCIIEncoding convertMess = new ASCIIEncoding();    //converts bytes to char/string for visual output
                        mess = convertMess.GetString(from);
                        //MessageBox.Show(mess);
                        test = mess.ToCharArray()[0];   //Grabs first letter of Message for while loop test... the full DCCMan was giving me grief to test
                    }
                    catch
                    {
                        MessageBox.Show("Problem obtaining DCC port state");
                    }
                }

                //MessageBox.Show(service.ToString());
                if (service == 4882) //DCC NOT READY SERVICE CODE
                {
                    MessageBox.Show(port + " was not able to be initialized correctly and interface will not function");
                    MessageBox.Show("Make sure you have the correct serial port number, or possibly change the port number (Device manager)");
                }
                else if (service == 4881)//DCC Ready service code
                {
                    MessageBox.Show(port + " was properly initialized: " + peterpiper.SafePipeHandle.DangerousGetHandle().ToString());
                }
            }
            else
                MessageBox.Show(port + " could not be initialized (TOS_InitDCC failed)");
        }
        //********************************end of dcc mgmnt**************************************

      
        //****************************TRACK STATUS MANAGEMENT***********************************
       private void timerCheck_Click(object sender, EventArgs e)
        {   //This initializes all the timer for the AIU and track updates
            try
            {
                if (timerCheck.Checked)
                {
                    timer1.Interval = 250;
                    timer1.Start();
                }
                else if (!timerCheck.Checked)
                {
                    timer1.Stop();
                }
            }
            catch (Exception ex)
            {MessageBox.Show("Failed to run timer" + ex.ToString());}

        }

        private void getMessButton_Click(object sender, EventArgs e)
        {   //Grabs a message sitting on Pipe
            byte[] from = new byte[255];
            uint service = 0;
            byte[] messagedata = new byte[16];
            uint messagelength = 0;
            int ret = 0;
            try
            {
                ret = GetMessageFromPeer(ref from[0], ref service, ref messagedata[0], ref messagelength, peterpiper.SafePipeHandle.DangerousGetHandle());
                ASCIIEncoding convertMess = new ASCIIEncoding();
                string mess = convertMess.GetString(from);

                ASCIIEncoding convertMess2 = new ASCIIEncoding();
                char[] mess2 = convertMess2.GetChars(messagedata);  //converts from bits to string/char[]
                //string mess2 = convertMess2.GetString(messagedata);

                MessageBox.Show(service.ToString());

                MessageBox.Show(mess.ToString());
                MessageBox.Show(((int)mess2[1]).ToString());
            }
            catch (Exception ex)
            {MessageBox.Show(ex.ToString());}
      }
  
        private void requestTrackUpdates_Click(object sender, EventArgs e)
        { timer1_Tick(sender, e);}  //an instance of the track 

        private void timer1_Tick(object sender, EventArgs e)
        {//this is the timer that polls the AIU's for updates
            int[] aiuList = new int[4];
            aiuList[0] = 60;    //the aius
            aiuList[1] = 61;
            aiuList[2] = 62;
            aiuList[3] = 63;

            //********again getting the messages back****************
            byte[] from = new byte[255];
            uint service = 0;
            int counter = 0;
            int ret = 0;
            byte[] messagedata = new byte[16];
            uint messagelength = 0;
            string newStatusMessage;
            //******************************************************

            for (int i = 0; i < aiuList.Length; i++)
            {TOS_ReadAIU((uint)aiuList[i], 1, peterpiper.SafePipeHandle.DangerousGetHandle());  }

            for (int i = 0; i < aiuList.Length; i++)
            {TOS_ReadAIU((uint)aiuList[i], 1, peterpiper.SafePipeHandle.DangerousGetHandle());}
            //Doing the above twice evens out the 'junk' that can build up when other cmds are being sent in the pipe


            while (ret > 0 || counter < 8)
            {
                counter++;
                service = 0;
                messagedata = new byte[16];
                messagelength = 0;

                try
                {
                    ret = GetMessageFromPeer(ref from[0], ref service, ref messagedata[0], ref messagelength, peterpiper.SafePipeHandle.DangerousGetHandle());
                    //getting message off the pipe of the AIU/track status

                    ASCIIEncoding convertMess = new ASCIIEncoding();
                    string mess = convertMess.GetString(from);

                    ASCIIEncoding convertMess2 = new ASCIIEncoding();
                    char[] mess2 = convertMess2.GetChars(messagedata);

                    for (int i = 0; i < aiuList.Length; i++)
                    {
                        if (((int)mess2[1]).ToString() == aiuList[i].ToString())
                        {
                            if (i == 0)
                            {
                                newStatusMessage = ((int)mess2[2]).ToString() + " " + ((int)mess2[3]).ToString() + " " + ((int)mess2[4]).ToString() + " " + ((int)mess2[5]).ToString();
                                //^^^^^^^^^ spits out the chars sent back from dccman as the aiu track status^^^^^^^^^^^^^^^^^

                                //MessageBox.Show(newStatusMessage + " :: " + aiuStatusMessage[i]); 
                                aiuText1.Text = newStatusMessage;
                                if (newStatusMessage != aiuStatusMessage[i])
                                {
                                    aiuStatusMessage[i] = newStatusMessage;
                                    aiuText1.BackColor = Color.Blue;    //if a change from last reading...BLUE
                                }
                                else
                                    aiuText1.BackColor = Color.White;   //and back white

                            }
                            if (i == 1)
                            {
                                newStatusMessage = ((int)mess2[2]).ToString() + " " + ((int)mess2[3]).ToString() + " " + ((int)mess2[4]).ToString() + " " + ((int)mess2[5]).ToString();
                                //MessageBox.Show(newStatusMessage + " :: " + aiuStatusMessage[i]); 
                                aiuText2.Text = newStatusMessage;
                                if (newStatusMessage != aiuStatusMessage[i])
                                {
                                    aiuStatusMessage[i] = newStatusMessage;
                                    aiuText2.BackColor = Color.Blue;
                                }
                                else
                                    aiuText2.BackColor = Color.White;
                            }
                            if (i == 2)
                            {
                                newStatusMessage = ((int)mess2[2]).ToString() + " " + ((int)mess2[3]).ToString() + " " + ((int)mess2[4]).ToString() + " " + ((int)mess2[5]).ToString();
                                //MessageBox.Show(newStatusMessage + " :: " + aiuStatusMessage[i]); 
                                aiuText3.Text = newStatusMessage;
                                if (newStatusMessage != aiuStatusMessage[i])
                                {
                                    aiuStatusMessage[i] = newStatusMessage;
                                    aiuText3.BackColor = Color.Blue;
                                }
                                else
                                    aiuText3.BackColor = Color.White;
                            }
                            if (i == 3)
                            {
                                newStatusMessage = ((int)mess2[2]).ToString() + " " + ((int)mess2[3]).ToString() + " " + ((int)mess2[4]).ToString() + " " + ((int)mess2[5]).ToString();
                                //MessageBox.Show(newStatusMessage + " :: " + aiuStatusMessage[i]); 
                                aiuText4.Text = newStatusMessage;
                                if (newStatusMessage != aiuStatusMessage[i])
                                {
                                    aiuStatusMessage[i] = newStatusMessage;
                                    aiuText4.BackColor = Color.Blue;
                                }
                                else
                                    aiuText4.BackColor = Color.White;
                            }
                        }
                    }
                }
                catch (Exception ex)
                { MessageBox.Show(ex.ToString());}
            }
        }
        //*******************************end of track status mgmt**************************

        //*****************************GamePad Management**********************************
        private void gamepadButton_Click(object sender, EventArgs e)
        {//this brings up the joystick control form.......
            stopButton_Click(sender, e);
            Joy joy = new Joy(Convert.ToUInt16(trainIdBox.Text), peterpiper.SafePipeHandle.DangerousGetHandle(), turnoutListBox);
            joy.Show();
        }
        //*******************************end of gamepad mgmt**********************************
    }  
   //**************************************End of Source*******************************************
}
